# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagCaracter
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE
from nfe.mdfe.manual_300 import *
import os

DIRNAME = os.path.dirname(__file__)


class InfDoc(XMLMDFe):
    """Informacoes dos documentos que serao inseridos no MDF-e."""
    def __init__(self):
        super(InfDoc, self).__init__()
        self.cMunDescarga = TagCaracter(nome=u'cMunDescarga', tamanho=[7, 7], raiz=u'//infDoc')
        self.xMunDescarga = TagCaracter(nome=u'xMunDescarga', tamanho=[2, 60], raiz=u'//infDoc')
        self.chNFe = TagCaracter(nome=u'chNFe', tamanho=[44, 44], raiz=u'//infDoc')

    @property
    def xml(self):
        xml = u'<infDoc>'
        xml += self.cMunDescarga.xml
        xml += self.xMunDescarga.xml
        xml += self.chNFe.xml
        xml += u'</infDoc>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cMunDescarga.xml = arquivo
        self.xMunDescarga.xml = arquivo
        self.chNFe.xml = arquivo


class EvIncDFeMDFe(XMLMDFe):
    """Leiaute especifico do evento de inclusao de DF-e."""
    def __init__(self):
        super(EvIncDFeMDFe, self).__init__()
        self.descEvento = TagCaracter(
            nome=u'descEvento', tamanho=[13, 13], raiz=u'/',
            valor=u'Inclusao DF-e')
        self.nProt = TagCaracter(nome=u'nProt', tamanho=[15, 15], raiz=u'//evIncDFeMDFe')
        self.cMunCarrega = TagCaracter(nome=u'cMunCarrega', tamanho=[7, 7], raiz=u'//evIncDFeMDFe')
        self.xMunCarrega = TagCaracter(nome=u'xMunCarrega', tamanho=[2, 60], raiz=u'//evIncDFeMDFe')
        self.infDoc = []
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'evInclusaoDFeMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = u'<evIncDFeMDFe xmlns="%s">' % NAMESPACE_MDFE
        xml += self.descEvento.xml
        xml += self.nProt.xml
        xml += self.cMunCarrega.xml
        xml += self.xMunCarrega.xml
        for i in self.infDoc:
            xml += i.xml
        xml += u'</evIncDFeMDFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.descEvento.xml = arquivo
            self.nProt.xml = arquivo
            self.cMunCarrega.xml = arquivo
            self.xMunCarrega.xml = arquivo
            self.infDoc = self.le_grupo(u'//evIncDFeMDFe/infDoc', InfDoc)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
