# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagCaracter, TagInteiro
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE
from nfe.mdfe.manual_300 import *
import os

DIRNAME = os.path.dirname(__file__)


class InfTermCarreg(XMLMDFe):
    """Grupo de informacoes dos terminais de carregamento"""
    def __init__(self):
        super(InfTermCarreg, self).__init__()
        self.cTermCarreg = TagCaracter(
            nome=u'cTermCarreg', tamanho=[1, 8], raiz=u'//aquav/infTermCarreg')
        self.xTermCarreg = TagCaracter(
            nome=u'xTermCarreg', tamanho=[1, 60], raiz=u'//aquav/infTermCarreg')

    @property
    def xml(self):
        xml = u'<infTermCarreg>'
        xml += self.cTermCarreg.xml
        xml += self.xTermCarreg.xml
        xml += u'</infTermCarreg>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cTermCarreg.xml = arquivo
        self.xTermCarreg.xml = arquivo


class InfTermDescarreg(XMLMDFe):
    """Grupo de informacoes dos terminais de descarregamento"""
    def __init__(self):
        super(InfTermDescarreg, self).__init__()
        self.cTermDescarreg = TagCaracter(
            nome=u'cTermDescarreg', tamanho=[1, 8], raiz=u'//aquav/infTermDescarreg')
        self.xTermDescarreg = TagCaracter(
            nome=u'xTermDescarreg', tamanho=[1, 60], raiz=u'//aquav/infTermDescarreg')

    @property
    def xml(self):
        xml = u'<infTermDescarreg>'
        xml += self.cTermDescarreg.xml
        xml += self.xTermDescarreg.xml
        xml += u'</infTermCarreg>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cTermDescarreg.xml = arquivo
        self.xTermDescarreg.xml = arquivo


class InfEmbComb(XMLMDFe):
    """Grupo de informacoes das embarcacoes do comboio"""
    def __init__(self):
        super(InfEmbComb, self).__init__()
        self.cEmbComb = TagCaracter(nome=u'cEmbComb', tamanho=[1, 10], raiz=u'//aquav/infEmbComb')
        self.xBalsa = TagCaracter(nome=u'xBalsa', tamanho=[1, 60], raiz=u'//aquav/infEmbComb')

    @property
    def xml(self):
        xml = u'<infEmbComb>'
        xml += self.cEmbComb.xml
        xml += self.xBalsa.xml
        xml += u'</infEmbComb>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cEmbComb.xml = arquivo
        self.xBalsa.xml = arquivo


class InfUnidCargaVazia(XMLMDFe):
    """Informacoes das unidades de cargas vazias"""
    def __init__(self):
        super(InfUnidCargaVazia, self).__init__()
        self.idUnidCargaVazia = TagCaracter(
            nome=u'idUnidCargaVazia', tamanho=[1, 20], raiz=u'//aquav/infUnidCargaVazia')
        self.tpUnidCargaVazia = TagInteiro(
            nome=u'tpUnidCargaVazia', tamanho=[1, 1, 1], raiz=u'//aquav/infUnidCargaVazia')

    @property
    def xml(self):
        xml = u'<infUnidCargaVazia>'
        xml += self.idUnidCargaVazia.xml
        xml += self.tpUnidCargaVazia.xml
        xml += u'</infUnidCargaVazia>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.idUnidCargaVazia.xml = arquivo
        self.tpUnidCargaVazia.xml = arquivo


class InfUnidTranspVazia(XMLMDFe):
    """Informacoes das unidades de cargas vazias"""
    def __init__(self):
        super(InfUnidTranspVazia, self).__init__()
        self.idUnidTranspVazia = TagCaracter(
            nome=u'idUnidTranspVazia', tamanho=[1, 20], raiz=u'//aquav/infUnidTranspVazia')
        self.tpUnidTranspVazia = TagInteiro(
            nome=u'tpUnidTranspVazia', tamanho=[1, 1, 1], raiz=u'//aquav/infUnidTranspVazia')

    @property
    def xml(self):
        xml = u'<infUnidTranspVazia>'
        xml += self.idUnidTranspVazia.xml
        xml += self.tpUnidTranspVazia.xml
        xml += u'</infUnidTranspVazia>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.idUnidTranspVazia.xml = arquivo
        self.tpUnidTranspVazia.xml = arquivo


class ModalAquav(XMLMDFe):
    """Informacoes do Modal Aereo"""
    def __init__(self):
        super(ModalAquav, self).__init__()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'mdfeModalAquaviario_v3.00.xsd'
        self.irin = TagCaracter(nome=u'irin', tamanho=[1, 10], raiz=u'//aquav')
        self.tpEmb = TagInteiro(nome=u'tpEmb', tamanho=[2, 2, 1], raiz=u'//aquav')
        self.cEmbar = TagCaracter(nome=u'cEmbar', tamanho=[1, 10], raiz=u'//aquav')
        self.xEmbar = TagCaracter(nome=u'xEmbar', tamanho=[1, 60], raiz=u'//aquav')
        self.nViag = TagInteiro(nome=u'nViag', tamanho=[1, 10, 1], raiz=u'//aquav')
        self.cPrtEmb = TagCaracter(nome=u'cPrtEmb', tamanho=[1, 5], raiz=u'//aquav')
        self.cPrtDest = TagCaracter(nome=u'cPrtDest', tamanho=[1, 5], raiz=u'//aquav')
        self.prtTrans = TagCaracter(
            nome=u'prtTrans', tamanho=[1, 60], raiz=u'//aquav', obrigatorio=False)
        self.tpNav = TagInteiro(
            nome=u'tpNav', tamanho=[1, 1, 1], raiz=u'//aquav', obrigatorio=False)
        self.infTermCarreg = []  # Nao obrigatorio - Maximo 5
        self.infTermDescarreg = []  # Nao obrigatorio - Maximo 5
        self.infEmbComb = []  # Nao obrigatorio - Maximo 30
        self.infUnidCargaVazia = []  # Nao obrigatorio
        self.infUnidTranspVazia = []  # Nao obrigatorio

    @property
    def xml(self):
        xml = u'<aquav xmlns="%s">' % NAMESPACE_MDFE
        xml += self.irin.xml
        xml += self.tpEmb.xml
        xml += self.cEmbar.xml
        xml += self.xEmbar.xml
        xml += self.nViag.xml
        xml += self.cPrtEmb.xml
        xml += self.cPrtDest.xml
        xml += self.tpNav.xml
        for itc in self.infTermCarreg[:5]:  # Nao obrigatorio - De 0 a 5 ocorrencias
            xml += itc.xml
        for itd in self.infTermDescarreg[:5]:  # Nao obrigatorio - De 0 a 5 ocorrencias
            xml += itd.xml
        for iec in self.infEmbComb[:30]:  # Nao obrigatorio - De 0 a 30 ocorrencias
            xml += iec.xml
        for iuc in self.infUnidCargaVazia:
            xml += iuc.xml
        for iut in self.infUnidTranspVazia:
            xml += iut.xml
        xml += u'</aquav>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.irin.xml = arquivo
        self.tpEmb.xml = arquivo
        self.cEmbar.xml = arquivo
        self.xEmbar.xml = arquivo
        self.nViag.xml = arquivo
        self.cPrtEmb.xml = arquivo
        self.cPrtDest.xml = arquivo
        self.tpNav.xml = arquivo
        self.infTermCarreg = self.le_grupo(u'//aquav/infTermCarreg', InfTermCarreg)
        self.infTermDescarreg = self.le_grupo(u'//aquav/infTermDescarreg', InfTermDescarreg)
        self.infEmbComb = self.le_grupo(u'//aquav/infEmbComb', InfEmbComb)
        self.infUnidCargaVazia = self.le_grupo(u'//aquav/infUnidCargaVazia', InfUnidCargaVazia)
        self.infUnidTranspVazia = self.le_grupo(u'//aquav/infUnidTranspVazia', InfUnidTranspVazia)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
