# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC, Signature
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE
from nfe.mdfe.manual_300 import *
import os

DIRNAME = os.path.dirname(__file__)


class InfProt(XMLMDFe):
    """Informacoes do protocolo de resposta."""
    def __init__(self):
        super(InfProt, self).__init__()
        self.Id = TagCaracter(nome=u'infProt', propriedade=u'Id', raiz=u'/')
        self.tpAmb = TagInteiro(nome=u'tpAmb', tamanho=[1, 1], raiz=u'//infProt')
        self.verAplic = TagCaracter(nome=u'verAplic', tamanho=[1, 20], raiz=u'//infProt')
        self.chMDFe = TagInteiro(nome=u'chMDFe', tamanho=[44, 44], raiz=u'//infProt')
        self.dhRecbto = TagDataHoraUTC(nome=u'dhRecbto', raiz=u'//infProt')
        self.nProt = TagCaracter(
            nome=u'nProt', tamanho=[15, 15], raiz=u'//infProt', obrigatorio=False)
        self.digVal = TagCaracter(
            nome=u'digVal', tamanho=[28, 28], raiz=u'//infProt', obrigatorio=False)
        self.cStat = TagCaracter(nome=u'cStat', tamanho=[3, 3], raiz=u'//infProt')
        self.xMotivo = TagCaracter(nome=u'xMotivo', tamanho=[1, 255], raiz=u'//infProt')
        self.signature = Signature()

    @property
    def xml(self):
        xml = u'<infProt Id="%s">' % self.Id.valor
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.chMDFe.xml
        xml += self.dhRecbto.xml
        xml += self.nProt.xml
        xml += self.digVal.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.signature.xml
        xml += u'</infProt>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.Id.xml = arquivo
        self.tpAmb.xml = arquivo
        self.verAplic.xml = arquivo
        self.chMDFe.xml = arquivo
        self.dhRecbto.xml = arquivo
        self.nProt.xml = arquivo
        self.digVal.xml = arquivo
        self.cStat.xml = arquivo
        self.xMotivo.xml = arquivo
        self.signature.xml = arquivo


class ProtMDFe(XMLMDFe):
    """Protocolo de recebimento do MDF-e."""
    def __init__(self):
        super(ProtMDFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'protMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade=u'versao',
            raiz=u'/', namespace=NAMESPACE_MDFE, valor=u'3.00')
        self.infProt = InfProt()

    @property
    def xml(self):
        xml = u'<protMDFe versao="%s" xmlns="%s">' % (self.versao.valor, NAMESPACE_MDFE)
        xml += self.infProt.xml
        xml += u'</protMDFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infProt.xml = arquivo


class RetConsReciMDFe(XMLMDFe):
    """Mensagem de retorno da consulta de processamento do MDFe transmitido."""
    def __init__(self):
        super(RetConsReciMDFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'retConsReciMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade=u'versao',
            raiz=u'/', namespace=NAMESPACE_MDFE, valor=u'3.00')
        self.tpAmb = TagInteiro(nome=u'tpAmb', tamanho=[1, 1], raiz=u'//retConsReciMDFe')
        self.verAplic = TagCaracter(nome=u'verAplic', tamanho=[1, 20], raiz=u'//retConsReciMDFe')
        self.nRec = TagInteiro(nome=u'nRec', tamanho=[15, 15], raiz=u'//retConsReciMDFe')
        self.cStat = TagCaracter(nome=u'cStat', tamanho=[3, 3], raiz=u'//retConsReciMDFe')
        self.xMotivo = TagCaracter(nome=u'xMotivo', tamanho=[1, 255], raiz=u'//retConsReciMDFe')
        self.cUF = TagInteiro(nome=u'cUF', tamanho=[2, 2], raiz=u'//retConsReciMDFe')
        self.protMDFe = ProtMDFe()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'retConsReciMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = u'<retConsReciMDFe xmlns="%s" versao="%s">' % (NAMESPACE_MDFE, self.versao.valor)
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.nRec.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.protMDFe.xml
        xml += u'</retConsReciMDFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.tpAmb.xml = arquivo
        self.verAplic.xml = arquivo
        self.nRec.xml = arquivo
        self.cStat.xml = arquivo
        self.xMotivo.xml = arquivo
        self.cUF.xml = arquivo
        self.protMDFe.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
