# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE
from nfe.mdfe.manual_300 import *
import os

DIRNAME = os.path.dirname(__file__)


class InfRec(XMLMDFe):
    """Número do recibo gerado pelo ambiente autorizador, composto por duas posicoes com o código
    da UF (codificacao do IBGE) onde foi entregue o arquivo"""
    def __init__(self):
        super(InfRec, self).__init__()
        self.nRec = TagInteiro(nome=u'nRec', tamanho=[15, 15], raiz=u'//infRec')
        self.dhRecbto = TagDataHoraUTC(nome=u'dhRecbto', raiz=u'//infRec')
        self.tMed = TagInteiro(nome=u'tMed', tamanho=[1, 4], raiz=u'//infRec')

    @property
    def xml(self):
        xml = u'<infRec>'
        xml += self.nRec.xml
        xml += self.dhRecbto.xml
        xml += self.tMed.xml
        xml += u'</infRec>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.nRec.xml = arquivo
        self.dhRecbto.xml = arquivo
        self.tMed.xml = arquivo


class RetEnviMDFe(XMLMDFe):
    """Mensagem de retorno do envio de MDF-e."""
    def __init__(self):
        super(RetEnviMDFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'retEnviMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade=u'versao',
            raiz=u'/', namespace=NAMESPACE_MDFE, valor=u'3.00')
        self.tpAmb = TagInteiro(nome=u'tpAmb', tamanho=[1, 1], raiz=u'//retEnviMDFe')
        self.cUF = TagInteiro(nome=u'cUF', tamanho=[2, 2], raiz=u'//retEnviMDFe')
        self.verAplic = TagCaracter(nome=u'verAplic', tamanho=[1, 20], raiz=u'//retEnviMDFe')
        self.cStat = TagCaracter(nome=u'cStat', tamanho=[3, 3], raiz=u'//retEnviMDFe')
        self.xMotivo = TagCaracter(nome=u'xMotivo', tamanho=[1, 255], raiz=u'//retEnviMDFe')
        self.infRec = InfRec()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'retEnviMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = u'<retEnviMDFe xmlns="%s" versao="%s">' % (NAMESPACE_MDFE, self.versao.valor)
        xml += self.tpAmb.xml
        xml += self.cUF.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.infRec.xml
        xml += u'</retEnviMDFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.tpAmb.xml = arquivo
        self.cUF.xml = arquivo
        self.verAplic.xml = arquivo
        self.cStat.xml = arquivo
        self.xMotivo.xml = arquivo
        self.infRec.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
