# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagCaracter, TagInteiro, TagDecimal, TagDataHoraUTC, Signature
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE
from nfe.pysped.xml_sped import ABERTURA
from nfe.mdfe.manual_300 import *
import os

DIRNAME = os.path.dirname(__file__)


class InfEvento(XMLMDFe):
    """Grupo de informacoes do registro do evento."""
    def __init__(self):
        super(InfEvento, self).__init__()
        self.Id = TagCaracter(nome=u'infEvento', tamnho=[48, 48], propriedade=u'Id', raiz=u'/')
        self.tpAmb = TagInteiro(nome=u'tpAmb', tamanho=[1, 1], raiz=u'//infEvento')
        self.verAplic = TagCaracter(nome=u'verAplic', tamanho=[1, 20], raiz=u'//infEvento')
        self.cOrgao = TagInteiro(nome=u'cOrgao', tamanho=[2, 2], raiz=u'//infEvento')
        self.cStat = TagCaracter(nome=u'cStat', tamanho=[3, 3], raiz=u'//infEvento')
        self.xMotivo = TagCaracter(nome=u'xMotivo', tamanho=[1, 255], raiz=u'//infEvento')
        self.chMDFe = TagInteiro(
            nome=u'chMDFe', tamanho=[44, 44], raiz=u'//infEvento', obrigatorio=False)
        self.tpEvento = TagInteiro(
            nome=u'tpEvento', tamanho=[6, 6], raiz=u'//infEvento', obrigatorio=False)
        self.xEvento = TagCaracter(
            nome=u'xEvento', tamanho=[5, 60], raiz=u'//infEvento', obrigatorio=False)
        self.nSeqEvento = TagInteiro(
            nome=u'nSeqEvento', tamanho=[1, 2], raiz=u'//infEvento', obrigatorio=False)
        self.dhRegEvento = TagDataHoraUTC(
            nome=u'dhRegEvento', raiz=u'//infEvento', obrigatorio=False)
        self.nProt = TagCaracter(
            nome=u'nProt', tamanho=[15, 15], raiz=u'//infEvento', obrigatorio=False)
        self.signature = Signature()

    @property
    def xml(self):
        xml = u'<infEvento Id="%s">' % self.Id.valor
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.chMDFe.xml
        xml += self.tpEvento.xml
        xml += self.xEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.dhRegEvento.xml
        xml += self.nProt.xml
        xml += u'</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.Id.xml = arquivo
        self.tpAmb.xml = arquivo
        self.verAplic.xml = arquivo
        self.cOrgao.xml = arquivo
        self.cStat.xml = arquivo
        self.xMotivo.xml = arquivo
        self.chMDFe.xml = arquivo
        self.tpEvento.xml = arquivo
        self.xEvento.xml = arquivo
        self.nSeqEvento.xml = arquivo
        self.dhRegEvento.xml = arquivo
        self.nProt.xml = arquivo


class RetEventoMDFe(XMLMDFe):
    """Mensagem de retorno do resultado da solicitacao de registro de evento do MDF-e."""
    def __init__(self):
        super(RetEventoMDFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'retEventoMDFe', tamanho=[1, 4, 1], decimais=[0, 2, 2], propriedade=u'versao',
            raiz=u'/', namespace=NAMESPACE_MDFE, valor=u'3.00')
        self.infEvento = InfEvento()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'retEventoMDFe_v3.00.xsd'

    @property
    def xml(self):
        xml = ABERTURA
        xml += u'<retEventoMDFe xmlns="%s" versao="%s">' % (NAMESPACE_MDFE, self.versao.valor)
        xml += self.infEvento.xml
        xml += u'</retEventoMDFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
