# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.pysped.xml_sped import TagInteiro, TagDecimal, tira_abertura, ABERTURA
from nfe.mdfe.base import XMLMDFe, NAMESPACE_MDFE


class MDFeCabecMsg(XMLMDFe):
    """Cabecalho (header) de mensagem de requisicao padrao SOAP."""
    def __init__(self):
        super(MDFeCabecMsg, self).__init__()
        self.servico = u''
        self.cUF = TagInteiro(
            nome=u'cUF', tamanho=[2, 2], raiz=u'//mdfeCabecMsg',  valor=31)
        self.versaoDados = TagDecimal(
            nome=u'versaoDados', tamanho=[1, 4], raiz=u'//mdfeCabecMsg', valor=u'3.00')

    @property
    def xml(self):
        xml = u'<mdfeCabecMsg xmlns="%s/wsdl/%s">' % (NAMESPACE_MDFE, self.servico)
        xml += self.cUF.xml
        xml += self.versaoDados.xml
        xml += u'</mdfeCabecMsg>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.cUF.xml = arquivo
        self.versaoDados.xml = arquivo


class MDFeDadosMsg(XMLMDFe):
    """Dados (data) de mensagem de requisicao padrao SOAP."""
    def __init__(self):
        super(MDFeDadosMsg, self).__init__()
        self.servico = u''
        self.dados = None

    @property
    def xml(self):
        xml = u'<mdfeDadosMsg xmlns="%s/wsdl/%s">' % (NAMESPACE_MDFE, self.servico)
        xml += tira_abertura(self.dados.xml)
        xml += u'</mdfeDadosMsg>'

        return xml


class SOAPEnvio(XMLMDFe):
    def __init__(self):
        super(SOAPEnvio, self).__init__()
        self.servico = u''
        self.cUF = None
        self.dados = None
        self.mdfeCabecMsg = MDFeCabecMsg()
        self.mdfeDadosMsg = MDFeDadosMsg()
        self._header = {u'content-type': u'application/soap+xml; charset=utf-8'}

    @property
    def xml(self):
        self.mdfeCabecMsg.servico = self.servico
        self.mdfeCabecMsg.cUF.valor = self.cUF
        self.mdfeCabecMsg.versaoDados.valor = self.dados.versao.valor
        self.mdfeDadosMsg.servico = self.servico
        self.mdfeDadosMsg.dados = self.dados
        self._header[u'content-type'] += u'; action="%s/wsdl/%s"' % (NAMESPACE_MDFE, self.servico)

        xml = ABERTURA
        xml += u'<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope">'
        xml += u'<soap:Header>'
        xml += self.mdfeCabecMsg.xml
        xml += u'</soap:Header>'
        xml += u'<soap:Body>'
        xml += self.mdfeDadosMsg.xml
        xml += u'</soap:Body>'
        xml += u'</soap:Envelope>'

        return xml

    @property
    def header(self):
        header = self._header

        return header


class SOAPRetorno(XMLMDFe):
    def __init__(self):
        super(SOAPRetorno, self).__init__()
        self.servico = u''
        self.metodo = u''
        self.mdfeCabecMsg = MDFeCabecMsg()
        self.resposta = None

    @property
    def xml(self):
        xml = ABERTURA
        xml += u'<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope">'
        xml += u'<soap:Header>'
        xml += u'<mdfeCabecMsg xmlns="%s/wsdl/%s">' % (NAMESPACE_MDFE, self.servico)
        xml += self.mdfeCabecMsg.xml
        xml += u'</mdfeCabecMsg>'
        xml += u'</soap:Header>'
        xml += u'<soap:Body>'
        xml += u'<%sResult xmlns="%s/wsdl/%s">' % (self.metodo, NAMESPACE_MDFE, self.servico)
        xml += self.resposta.xml
        xml += u'</%sResult>' % self.metodo
        xml += u'</soap:Body>'
        xml += u'</soap:Envelope>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.mdfeCabecMsg.xml = arquivo
        self.resposta.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
