# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.mdfe.manual_300.processador import ProcessadorMDFe
from nfe.mdfe.manual_300.MDFe import InfMunCarrega, InfMunDescarga, InfNFe
from nfe.mdfe.manual_300.modalRodoviario import Condutor
from nfe.mdfe.manual_300.MDFe import MDFe
from datetime import datetime
import sys
if sys.version_info.major == 3:
    unicode = str

def _mod11(sequencia):
    """
    Realiza o calculo do modulo 11. O módulo 11 eh calculado multiplicando-se cada algarismo
    pela sequencia de multiplicadores 2,3,4,5,6,7,8,9,2,3, ... posicionados da direita para a
    esquerda.

    :param sequencia: sequencia de numeros utilizada no calculo do modulo-11
    :type sequencia: int|long|string
    :return: digito verificador
    :rtype: int

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    sequencia = ''.join(filter(lambda f: f in '0123456789', str(sequencia)))  # Algarismos
    soma = 0
    peso = 2
    for nr in reversed(sequencia):
        soma += int(nr) * peso
        peso += 1
        if peso > 9:
            peso = 2
    result = 11 - soma % 11
    result = 0 if result > 9 else result

    return result


cert = open('cert').read()
key = open('key').read()
AMBIENTE_HOMOLOGACAO = 2
MG = 31

mdfe = MDFe()
mdfe.infMDFe.tot.qCarga.valor = 0
mdfe.infMDFe.tot.cUnid.valor = u'01'
mdfe.infMDFe.tot.vCarga.valor = 0
mdfe.infMDFe.tot.qNFe.valor = 1
# mdfe.infMDFe.tot.qNFe.valor = 0
infMunDescarga = InfMunDescarga()  # Informacoes da carga -------+---------.
infMunDescarga.xMunDescarga.valor = u'Sao Jose dos Campos'  # <--.       |
infMunDescarga.cMunDescarga.valor = u'3549904'  # <--------------'       |
# infMunDescarga.xMunDescarga.valor = u'Sete Lagoas'  # <----------.         |
# infMunDescarga.cMunDescarga.valor = u'3167202'  # <--------------'         |
infNFe = InfNFe()  # Dados da NF-e  (pode ser CT-e, NF-e, ou NF mod. 01)   |
infNFe.chNFe.valor = u'31190101526558000178552650000020411000000185'  # <--|
infMunDescarga.infNFe.append(infNFe)  # <--------------------------------|
mdfe.infMDFe.infDoc.infMunDescarga.append(infMunDescarga)  # <-------------'
mdfe.infMDFe.emit.enderEmit.uf.valor = u'MG'
mdfe.infMDFe.emit.enderEmit.xMun.valor = u'Sete Lagoas'
mdfe.infMDFe.emit.enderEmit.cMun.valor = u'3167202'
mdfe.infMDFe.emit.enderEmit.xBairro.valor = u'Jardim Arizona'
mdfe.infMDFe.emit.enderEmit.nro.valor = u'700'
mdfe.infMDFe.emit.enderEmit.xLgr.valor = u'R. Joao Pereira Amorim'
mdfe.infMDFe.emit.xNome.valor = u'Sig Solucoes em Informatica LTDA'
mdfe.infMDFe.emit.ie.valor = u'0016381470089'
mdfe.infMDFe.emit.cnpj.valor = u'01526558000178'

infMunCarrega = InfMunCarrega()
infMunCarrega.xMunCarrega.valor = u'Sete Lagoas'
infMunCarrega.cMunCarrega.valor = u'3167202'
mdfe.infMDFe.ide.infMunCarrega.append(infMunCarrega)
mdfe.infMDFe.ide.cUF.valor = 31
mdfe.infMDFe.ide.tpAmb.valor = 2
mdfe.infMDFe.ide.tpEmit.valor = 2  # 1= Prestador de servico de transporte / 2= Transporte proprio
mdfe.infMDFe.ide.serie.valor = 134
mdfe.infMDFe.ide.nMDF.valor = 1  # Nr. do MDF-e
mdfe.infMDFe.ide.cMDF.valor = u'12345678'  # ID do registro no banco de dados

# mdfe.infMDFe.ide.indCarregaPosterior.valor = 1  # Para inserir novos carregamentos (municipios de carga e descarga serao iguais)
mdfe.infMDFe.ide.modal.valor = 1  # ModalRodo ----------------------------+-.
mdfe.infMDFe.infModal.modalRodo.veicTracao.placa.valor = u'XYZ1234'  # <--| |
mdfe.infMDFe.infModal.modalRodo.veicTracao.tara.valor = 5000  # <---------| |
mdfe.infMDFe.infModal.modalRodo.veicTracao.tpRod.valor = u'06'  # <-------| |
mdfe.infMDFe.infModal.modalRodo.veicTracao.tpCar.valor = u'00'  # <-------| |
mdfe.infMDFe.infModal.modalRodo.veicTracao.uf.valor = u'MG'  # <----------´ |
condutor1 = Condutor()  #  Primeiro condutor * obrigatorio *                |
condutor1.cpf.valor = u'37371541120'  #                                     |
condutor1.xNome.valor = u'Pierre Geraldo Moura Barbosa'  #                  |
mdfe.infMDFe.infModal.modalRodo.veicTracao.condutor.append(condutor1)  # <--'
mdfe.infMDFe.infModal.modalRodo.validar()

mdfe.infMDFe.ide.dhEmi.valor = u'2019-11-07T10:05:26 -03:00'
mdfe.infMDFe.ide.tpEmis.valor = 1  # Normal
mdfe.infMDFe.ide.procEmi.valor = u'0'
mdfe.infMDFe.ide.verProc.valor = u'SIG-ERP 9J1'
mdfe.infMDFe.ide.ufIni.valor = u'MG'
mdfe.infMDFe.ide.ufFim.valor = u'SP'
# mdfe.infMDFe.ide.ufFim.valor = u'MG'
chave_acesso = str(MG) + str(datetime.now().year)[-2:] + str(datetime.now().month).zfill(2) + \
               mdfe.infMDFe.emit.cnpj.valor + u'58' + str(mdfe.infMDFe.ide.serie.valor).zfill(3) + \
               str(mdfe.infMDFe.ide.nMDF.valor).zfill(9) + str(mdfe.infMDFe.ide.tpEmis.valor) + \
               mdfe.infMDFe.ide.cMDF.valor.zfill(8)
mdfe.infMDFe.ide.cDV.valor = str(_mod11(chave_acesso))  # Digito verificador (modulo 11)
mdfe.infMDFe.Id.valor = u'MDFe' + chave_acesso + str(mdfe.infMDFe.ide.cDV.valor)

mdfe.infMDFeSupl.qrCodMDFe.valor = u'https://dfe-portal.svrs.rs.gov.br/mdfe/qrCode?chMDFe='
mdfe.infMDFeSupl.qrCodMDFe.valor += chave_acesso + mdfe.infMDFe.ide.cDV.valor
mdfe.infMDFeSupl.qrCodMDFe.valor += u'&tpAmb=' + str(AMBIENTE_HOMOLOGACAO)
p = ProcessadorMDFe(cert, key, AMBIENTE_HOMOLOGACAO, MG).enviar_lote(mdfe)

# Registro do numero do recibo
if not p.resposta.alertas:
    open(mdfe.infMDFe.Id.valor, 'a+').write(
        '%s: %s - %s\n' % (p.resposta.infRec.dhRecbto.valor,
                           p.resposta.infRec.nRec.valor, p.resposta.xMotivo.valor))

print('Lote para consulta: %s' % p.resposta.infRec.nRec.valor)
