# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.nfe_400.processador import ProcessadorNFe, DANFE
from nfe.nfe_400.manual_600.nfe_400 import NFe as NFe_400
from nfe.nfe_400.manual_600.consrecinfe import ProtNFe, ProcNFe
from nfe.nfe_400.manual_600.cancnfe import RetCancNFe
from StringIO import StringIO
from xml.dom import minidom
from nfe.pysped.nfe.danfe.danferetrato import *
import pytz
from datetime import datetime
from nfe.nf_e import nf_e as nfe_310

LOCAL_TZ = pytz.timezone('America/Sao_Paulo')


class nfe_400(nfe_310):
    """
    Nota Fiscal Eletronica
    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """
    def __init__(self, certify=None, key=None, environment=2, state=u'', contingency=u''):
        super(nfe_400, self).__init__()
        self._certify = certify
        self._key = key
        self._environment = environment
        self._state = state
        self._contingency = contingency
        self._version = u'4.00'
        self.process = ProcessadorNFe(certify, key, environment, state, contingency)
        self.danfe = DANFE()

    @staticmethod
    def extrair_certificado_a1(fname, passwd):
        return super(nfe_400).extrair_certificado_a1(fname, passwd)

    def processar_notas(self, xml, *args):
        """
        Transmite e processa as notas fiscais eletronicas.

        :param xml: lista de xml das notas fiscais
        :type xml: list
        :return: dados do processo e arquivos de retorno.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        lista_nfe = list()
        for xml in xml if isinstance(xml, list) else [xml]:
            nfe = NFe_400()
            nfe.xml = xml
            lista_nfe.append(nfe)

        return self.process.processar_notas(lista_nfe)

    def consultar_recibo(self, lote, *args):
        """
        Realiza a consulta ao lote processado de notas fiscais.
        
        :param lote: o numero do lote (recibo) para a consulta
        :type lote: str
        :return: dados do processo e arquivos de retorno.
        :rtype: tuple
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        return self.process.consultar_recibo(lote)

    def cancelar_nota(self, chave, protocolo, justificativa, *args):
        """
        Realiza o cancelamento da nota fiscal eletronica.
        
        :param chave:chave da nfe
        :param protocolo: protocolo do processamento da nfe
        :param justificativa: justificativa do cancelamento
        :return: dados do processo e arquivos de retorno.
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        return self.process.cancelar_nota(chave, protocolo, justificativa)

    def inutilizar_faixa_numeracao(self, ano, cnpj, serie, inicio, fim, justificativa, *args):
        """
        Realiza a inutilizacao de numeros de notas fiscais.
        
        :param ano: o ano relacionado a faixa de numeracao a ser inutilizada
        :param cnpj: o CNPJ da empresa
        :param serie: a serie utilizada para a faixa de numeros das notas fiscais
        :param inicio: o correspondente ao valor inicial da faixa de numeracao
        :param fim: o correspondente ao valor final da faixa de numeracao 
        :param justificativa: justificativa da inutilização
        :type ano: str
        :type cnpj: str
        :type serie: str
        :type inicio: int
        :type fim: int
        :return: dados do processo e arquivos de retorno.
        :rtype: dict
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        return self.process.inutilizar_nota(ano, cnpj, serie, inicio, fim, justificativa)

    def gerar_danfe(self, xml, retcan_nfe=None, site_emitente=u'', logo=u'',
                    nome_sistema=u'SigERP - www.sigsolucoes.net.br', leiaute_logo_vertical=False):
        """
        DANFE em arquivo PDF
        :param xml: 
        :param retcan_nfe: 
        :param site_emitente: 
        :param logo: 
        :param nome_sistema: 
        :param leiaute_logo_vertical: 
        :type xml: 
        :type retcan_nfe: 
        :type site_emitente: 
        :type logo: 
        :type nome_sistema: 
        :type leiaute_logo_vertical: 
        :return: 
        :rtype: 
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        danfe = DANFE()
        nota = NFe_400()
        prot_nfe = ProtNFe()
        proc = ProcNFe()
        canc_nfe = RetCancNFe()
        nota.xml = xml
        xml = minidom.parseString(xml.encode('utf-8'))
        xml = xml.getElementsByTagName("protNFe")[0]
        prot_nfe.xml = xml.toxml()
        proc.protNFe = prot_nfe

        # Danfe de nota fiscal cancelada
        if retcan_nfe:
            xml = minidom.parseString(retcan_nfe.encode('utf-8'))
            xml = xml.getElementsByTagName('retEvento')[0]
            resposta = xml.toxml()
            canc_nfe.xml = resposta

        danfe.NFe = nota
        danfe.protNFe = prot_nfe
        danfe.retCancNFe = canc_nfe
        danfe.obs_impressao = u'DANFE gerado em %s' % self.utc_to_local(datetime.now()).strftime(
            u'%d/%m/%Y as %H:%M:%S')
        danfe.nome_sistema = nome_sistema
        danfe.site = site_emitente
        danfe.logo = logo
        danfe.leiaute_logo_vertical = leiaute_logo_vertical
        danfe.gerar_danfe()
        danfe_pdf = StringIO()
        danfe.danfe.generate_by(PDFGenerator, filename=danfe_pdf)
        danfe.danfe = danfe_pdf.getvalue()
        danfe_pdf.close()

        return danfe.danfe

    @staticmethod
    def utc_to_local(utc_dt):
        local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(LOCAL_TZ)
        return LOCAL_TZ.normalize(local_dt)

    def consultar_nota(self, chave, *args):
        """
        Consulta da situacao atual da NF-e na Base de Dados do Portal Secretaria de Fazenda 
        Estadual.
        :param chave: chave de identificacao da nota fiscal
        :type chave: str
        :return: dados do processo e arquivos de retorno
        :rtype: basestring

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        return self.process.consultar_nota(chave)

    def consultar_cadastro_contribuinte(self, cpf_cnpj, inscricao_estadual, *args):
        """
        Consulta dados de cadastro do contribuinte.
        
        :param cpf_cnpj: CPF/CNPJ do contribuinte
        :param inscricao_estadual: numero da Inscricao Estadual
        :type cpf_cnpj: str
        :type inscricao_estadual: str
        :return: 
        :rtype: dados do processo e arquivos de retorno

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        return self.process.consultar_cadastro_contribuinte(cpf_cnpj,  inscricao_estadual)

    def efetuar_manifesto(self, cnpj, evento, chave, motivo, *args):
        """
        Manifestacao do destinatario
        """

        return self.process.manifesto_destinatario(cnpj, evento, chave, motivo)

    def consultar_notas_destinatario(self, cnpj, nsu, *args):
        """
        Consulta manifestacao do destinatário
        """

        return self.process.consultar_notas_destinatario(cnpj, nsu)

    def download_xml(self, cnpj, chave, *args):
        """
        Baixa do arquivo XML da nota fiscal eletronica.

        :param cnpj: o CNPJ do emitente
        :param chave: a chave da nota fiscal eletronica
        :type cnpj: str
        :type chave: str
        :return: dados do processo e arquivos de retorno.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        return self.process.download_xml(cnpj, chave)

    def emitir_carta_correcao(self, chave, cnpj, correcao, seq=1, *args):
        """
        Emite carta de correcao.
        
        :param chave: a chave da nota fiscal eletronica
        :param cnpj: o CNPJ do emitente
        :param correcao: texto de correcao
        :param seq: seguencia crescente da carta de correcao
        :return: dados do processo e arquivos de retorno.
        :rtype: list
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        return self.process.carta_correcao(chave, cnpj, correcao, seq)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
