# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.nfce.danfe import DANFENfce, DANFENfceSimples
from nfe.nfce.processador import ProcessadorNFCe
from nfe.nfce.retenvcancnfce import ProcEventoNFe
from geraldo.generators import PDFGenerator
from nfe.nfce.consrecinfce import ProcNFCe
from nfe.nf_e400 import nfe_400
from StringIO import StringIO
import pytz

LOCAL_TZ = pytz.timezone('America/Sao_Paulo')


class nfce(nfe_400):
    """
    Nota fiscal de consumidor eletronica
    """
    def __init__(self, certify=None, key=None, environment=2, state=u'', contingency=u''):
        super(nfce, self).__init__(certify, key, environment, state, contingency)
        self.process = ProcessadorNFCe(certify, key, environment, state, contingency)

    def danfe_nfce(self, xml_auth, xml_canc, logo=u'', **kwargs):
        """
        DANFE NFC-e em arquivo PDF

        :param xml: estrutura XML da nota fiscal
        :param logo: logotipo da empresa (img - base64)
        :param leiaute: tipo do leiaute a ser utilizado na impressao do DANFE-NFCe, com o QR Code
        alinhado aa esquerda (padrao) ou centralizado
        :param resumido: DANFE-NFCe em formato resumido (sem os itens da nota) ou completo
        :type xml: str
        :type logo: basestring
        :type leiaute: str
        :type resumido: bool
        :return: formulario com link de download do arquivo (DANFE-NFCe) em formato PDF.
        :rtype: object
        """
        procnfce = ProcNFCe()
        procnfce.xml = xml_auth
        cancnfce = ProcEventoNFe()
        cancnfce.xml = xml_canc or None
        danfe = DANFENfce(queryset=[procnfce], cancnfce=cancnfce, logo=logo, **kwargs)
        danfe.band_page_header = danfe.header
        danfe.band_detail = danfe.detail
        danfe.band_page_footer = danfe.footer
        pdf = StringIO()
        danfe.generate_by(PDFGenerator, filename=pdf)
        danfenfce = pdf.getvalue()
        pdf.close()

        return danfenfce

    def consultar_nota(self, chave, *args):
        """
        Consulta da situacao atual da NF-e na Base de Dados do Portal Secretaria de Fazenda
        Estadual.
        :param chave: chave de identificacao da nota fiscal
        :type chave: str
        :return: dados do processo e arquivos de retorno
        :rtype: basestring

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        return self.process.consultar_nota(chave)
    
    def danfe_simples_nfce(self, xml, logo=u'', leiaute='qr_code_left', resumido=False):
        """
        DANFE NFC-e simplificado em arquivo PDF

        :param xml: estrutura XML da nota fiscal
        :param logo: logotipo da empresa (img - base64)
        :param leiaute: tipo do leiaute a ser utilizado na impressao do DANFE-NFCe, com o QR Code
        alinhado aa esquerda (padrao) ou centralizado
        :param resumido: DANFE-NFCe em formato resumido (sem os itens da nota) ou completo
        :type xml: str
        :type logo: basestring
        :type leiaute: str
        :type resumido: bool
        :return: formulario com link de download do arquivo (DANFE-NFCe) em formato PDF.
        :rtype: object
        """
        procnfce = ProcNFCe()
        procnfce.xml = xml
        danfe = DANFENfceSimples(queryset=[procnfce])
        danfe.band_page_header = danfe.header
        danfe.band_detail = danfe.detail
        danfe.band_page_footer = danfe.footer
        pdf = StringIO()
        danfe.generate_by(PDFGenerator, filename=pdf)
        danfenfc = pdf.getvalue()
        pdf.close()

        return danfenfc

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
