# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2018 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2018, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.nfe_400.manual_600.consrecinfe import ProcNFe
from geraldo import Report, SubReport, FIELD_ACTION_COUNT, RoundRect
from dateutil import tz
from base import *
import locale
locale.setlocale(locale.LC_ALL, 'pt_BR.utf8')


class DANFENfce(Report):
    def __init__(self, queryset=None, cancnfce=None, logo=None, layout='left', **kwargs):
        super(DANFENfce, self).__init__(queryset)
        self.print_if_empty = True
        self.additional_fonts = FONTS
        self.page_size = kwargs.get('page_size') or PAGE_SIZE  # default = A5
        self.margin_top = MARGIN_TOP
        self.margin_bottom = MARGIN_BOTTOM
        self.margin_left = MARGIN_LEFT
        self.margin_right = MARGIN_RIGHT
        self.qrcode = queryset[0].NFe.infNFeSupl.qrCode.valor
        _contingency = self._check_contingency()
        _cancel = cancnfce and cancnfce.retEvento.infEvento.Id.valor
        _logo = logo

        # Divisoes do DANFE NFC-e
        self.header = Header(_logo)
        self.detail = Detail()
        self.items = Items()
        self.layout = (layout == 'left' and LayoutLeft(self.qrcode, _contingency, _cancel) or
                       LayoutCenter(self.qrcode, _contingency, _cancel))
        self.layout.visible = True
        self.payment = Payment()
        self.payment.band_footer.child_bands = self.layout,
        self.detail.elements.append(self.items)
        self.detail.elements.append(self.payment)
        self.footer = Footer()

    # Contingencia
    def _check_contingency(self):
        """Verifica se nota fiscal foi emitida em contingencia"""
        if isinstance(self.queryset[0], ProcNFe):
            self.band_begin.visible = self.queryset[0].NFe.infNFe.Id.valor[37] == '9'
            return self.band_begin.visible

        return False

    # noinspection PyProtectedMember
    def do_on_new_page(self, page, page_number, generator):
        if generator._current_page_number != 1:
            self.band_page_footer = generator._is_latest_page and self.footer

    def format_date(self, data, formato):
        if not data.tzinfo:
            data = data.replace(tzinfo=tz.gettz('UTC')).astimezone(tz.tzlocal())
        return data.strftime(formato.encode('utf-8')).decode('utf-8')

    # noinspection PyPep8Naming
    class band_begin(BandDANFE):
        def __init__(self):
            super(DANFENfce.band_begin, self).__init__()
            self.borders = dict(all=True)
            self.height = 1*cm
            self.visible = False  # Sera mostrado somente em contingencia
            self.add_label(u'EMITIDA EM CONTINGÊNCIA', top=0, left=0, width=13.8,
                           style=OCCURRENCE_CENTER_13)
            self.add_label(u'Pendente de autorização', top=.36 * 1.5, left=0, width=13.8,
                           style=OCCURRENCE_CENTER)


class Header(BandDANFE):
    def __init__(self, logo=None):
        super(Header, self).__init__()
        self.height = 1.45*cm
        # self.borders = dict(all=RoundRect(stroke_color=YELLOW))
        _logo = isinstance(logo, StringIO) and logo or logo and StringIO(LOGO.decode('base64'))
        _left = 0
        if _logo and _logo.len:
            self.add_image(_logo, top=0, left=0, width=2.5, height=2.5)
            _left = 1.6  # Ajusta a distancia do cabecalho em funcao da logomarca
        fld = self.add_field('NFe.infNFe.emit.CNPJ.valor', top=0, left=_left, width=13.8 - _left,
                             style=not _left and DEFAULT_CENTER or DEFAULT)
        fld.get_value = lambda i: 'CNPJ {}.{}.{}/{}-{} <strong>{}</strong>'.format(
            *[str(i.NFe.infNFe.emit.CNPJ.valor).zfill(14)[x:x + y] for x, y in zip(
                (0, 2, 5, 8, 12), (2, 3, 3, 4, 2))] + [i.NFe.infNFe.emit.xNome.valor])
        fld = self.add_field('NFe.infNFe.emit.enderEmit.xLgr.valor', top=.36, left=_left,
                             width=13.8 - _left, style=not _left and DEFAULT_CENTER or DEFAULT)
        fld.get_value = lambda instance: ', '.join([
            instance.NFe.infNFe.emit.enderEmit.xLgr.valor,
            instance.NFe.infNFe.emit.enderEmit.nro.valor,
            instance.NFe.infNFe.emit.enderEmit.xBairro.valor, '-'.join([
                instance.NFe.infNFe.emit.enderEmit.xMun.valor,
                instance.NFe.infNFe.emit.enderEmit.UF.valor])])
        self.add_label('Documento Auxiliar da Nota Fiscal de Consumidor Eletrônica',
                       top=.36 * 2, left=_left, width=13.8 - _left,
                       style=not _left and DEFAULT_CENTER or DEFAULT)


class Detail(BandDANFE):
    def __init__(self):
        super(Detail, self).__init__()
        self.height = 16.2*cm
        # self.borders = dict(all=RoundRect(stroke_color=MAROON))


class Footer(BandDANFE):
    def __init__(self):
        super(Footer, self).__init__()
        self.height = 1.3*cm
        # self.borders = dict(all=RoundRect(stroke_color=PURPLE))
        self.add_field('NFe.infNFe.infAdic.infCpl.valor', top=.1, left=0, width=13.8,
                       style=ITEM_CENTER)

class Items(SubReport):
    def __init__(self):
        super(Items, self).__init__()
        self.get_queryset = lambda cls, parent: parent.NFe.infNFe.det

    # noinspection PyPep8Naming
    class band_header(BandDANFE):
        def __init__(self):
            super(Items.band_header, self).__init__()
            self.height = .36*cm
            # self.borders = dict(all=RoundRect(stroke_color=RED))
            self.add_label('Código', top=0, left=0, width=2, style=ITEM_BOLD)
            self.add_label('Descrição', top=0, left=2, width=5.5, style=ITEM_BOLD)
            self.add_label('Qtde.', top=0, left=7.5, width=1.5, style=ITEM_BOLD_RIGHT)
            self.add_label('UN', top=0, left=9.3, width=1, style=ITEM_BOLD)
            self.add_label('Vl Unit', top=0, left=10, width=2, style=ITEM_BOLD_RIGHT)
            self.add_label('Vl Total', top=0, left=12, width=1.8, style=ITEM_BOLD_RIGHT)

    # noinspection PyPep8Naming
    class band_detail(BandDANFE):
        def __init__(self):
            super(Items.band_detail, self).__init__()
            self.height = .4*cm
            # self.borders = dict(all=RoundRect(stroke_color=GREEN))
            fld = self.add_field('prod', top=0, left=0, width=2, style=ITEM)
            fld.get_value = \
                lambda i: i.prod.cEAN.valor.isdigit() and i.prod.cEAN.valor or i.prod.cProd.valor
            self.add_field('prod.xProd.valor', top=0, left=2, width=5.5, style=ITEM)
            fld = self.add_field('prod.qCom.valor', top=0, left=7.5, width=1.5, style=ITEM_RIGHT)
            fld.get_value = lambda i: (i.prod.qCom.valor - int(i.prod.qCom.valor)) and \
                                      ('%.3f' % float(i.prod.qCom.valor)).replace(
                                          '.', ',') or int(i.prod.qCom.valor)
            self.add_field('prod.uCom.valor', top=0, left=9.3, width=1, style=ITEM)
            fld = self.add_field('prod.vUnCom.valor', top=0, left=10, width=2, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.prod.vUnCom.valor), symbol=False, grouping=True)
            fld = self.add_field('prod.vProd.valor', top=0, left=12, width=1.8, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.prod.vProd.valor), symbol=False, grouping=True)
            self.auto_expand_height = True

    # noinspection PyPep8Naming
    class band_footer(BandDANFE):
        def __init__(self):
            super(Items.band_footer, self).__init__()
            self.height = 2.3*cm
            # self.borders = dict(all=RoundRect(stroke_color=BLUE))
            self.add_label('Quantidade total de itens', top=.36, left=0, width=6)
            fld = self.add_field('prod.cProd.valor', top=.36, left=0, width=13.8, style=ITEM_RIGHT)
            fld.action = FIELD_ACTION_COUNT
            self.add_label('Valor total R$', top=.36 * 2, left=0, width=6)
            fld = self.add_field('NFe.infNFe.total.ICMSTot.vProd.valor', top=.36 * 2, left=0,
                                 width=13.8, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.NFe.infNFe.total.ICMSTot.vProd.valor), symbol=False, grouping=True)
            self.add_label('Desconto R$', top=.36 * 3, left=0, width=6)
            fld = self.add_field('NFe.infNFe.total.ICMSTot.vDesc.valor', top=.36 * 3, left=0,
                                 width=13.8, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.NFe.infNFe.total.ICMSTot.vDesc.valor) * -1, symbol=False, grouping=True)
            self.add_label('Seguro R$', top=.36 * 4, left=0, width=6)
            fld = self.add_field('NFe.infNFe.total.ICMSTot.vSeg.valor', top=.36 * 4, left=0,
                                 width=13.8, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.NFe.infNFe.total.ICMSTot.vSeg.valor), symbol=False, grouping=True)
            self.add_label('Frete R$', top=.36 * 6, left=0, width=6)
            fld = self.add_field('NFe.infNFe.total.ICMSTot.vFrete.valor', top=.36 * 6, left=0,
                                 width=13.8, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.NFe.infNFe.total.ICMSTot.vFrete.valor), symbol=False, grouping=True)
            self.add_label('Outros R$', top=.36 * 5, left=0, width=6)
            fld = self.add_field('NFe.infNFe.total.ICMSTot.vOutro.valor', top=.36 * 5, left=0,
                                 width=13.8, style=ITEM_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.NFe.infNFe.total.ICMSTot.vOutro.valor), symbol=False, grouping=True)
            self.add_label('Valor a pagar R$', top=.36 * 7, left=0, width=6, style=OCCURRENCE_8B)
            fld = self.add_field('NFe.infNFe.total.ICMSTot.vNF.valor', top=.36 * 7, left=0,
                                 width=13.8, style=OCCURRENCE_8B_RIGHT)
            fld.get_value = lambda i: locale.currency(float(
                i.NFe.infNFe.total.ICMSTot.vNF.valor), symbol=False, grouping=True)
            self.auto_expand_height = True


class Payment(SubReport):  # Pagamentos (formas)
    def __init__(self):
        super(Payment, self).__init__()
        self.get_queryset = lambda cls, parent: parent.NFe.infNFe.pag.detPag

    # noinspection PyPep8Naming
    class band_header(BandDANFE):
        def __init__(self):
            super(Payment.band_header, self).__init__()
            self.height = .4*cm
            # self.borders = dict(all=RoundRect(stroke_color=NAVY))
            self.add_label('FORMAS DE PAGAMENTO', top=.36, left=0, width=6, style=DEFAULT)
            self.add_label('VALOR PAGO R$', top=.36, left=0, width=13.8, style=DEFAULT_RIGHT)
            self.auto_expand_height = True

    # noinspection PyPep8Naming
    class band_detail(BandDANFE):
        def __init__(self):
            self.height = .4*cm
            # self.borders = dict(all=RoundRect(stroke_color=AQUA, radius=.8))

            # Formas de pagamento NFC-e
            _pay = {
                '01': u'Dinheiro',
                '02': u'Cheque',
                '03': u'Cartão de Crédito',
                '04': u'Cartão de Débito',
                '05': u'Crédito da Loja',
                '10': u'Vale Alimentação',
                '11': u'Vale Refeição',
                '12': u'Vale Presente',
                '13': u'Vale Combustível',
                '99': u'Outras'
            }
            super(Payment.band_detail, self).__init__()
            fld = self.add_field('tPag.valor', top=0, left=0, width=13.8, style=DEFAULT)
            fld.get_value = lambda i: _pay.get(i.tPag.valor, 'Outros')
            fld = self.add_field('vPag.valor', top=0, left=0, width=13.8, style=DEFAULT_RIGHT)
            fld.get_value = lambda i: locale.currency(
                float(i.vPag.valor), symbol=False, grouping=True)
            self.auto_expand_height = True

    # noinspection PyPep8Naming
    class band_footer(BandDANFE):
        def __init__(self):
            super(Payment.band_footer, self).__init__()
            self.height = 1.7*cm
            # self.borders = dict(all=RoundRect(stroke_color=NAVY))
            self.add_label('Troco', top=0, left=0, width=13.8, style=DEFAULT)
            fld = self.add_field('NFe.infNFe.pag.vTroco.valor', top=0, left=0, width=13.8,
                                 style=DEFAULT_RIGHT)
            fld.get_value = lambda i: locale.currency(
                float(i.NFe.infNFe.pag.vTroco.valor), symbol=False, grouping=True)
            self.add_label('Consulta pela chave de acesso', top=.36 * 2, left=0, width=13.8,
                           style=DEFAULT_BOLD_CENTER)
            self.add_field('NFe.infNFeSupl.urlChave.valor', top=.36 * 3, left=0, width=13.8,
                           style=DEFAULT_CENTER)
            fld = self.add_field('NFe.infNFe.Id.valor', top=.36 * 4, left=0, width=13.8,
                                 style=DEFAULT_CENTER)
            fld.get_value = lambda i: '{} {} {} {} {} {} {} {} {} {} {}'.format(
                *[str(i.NFe.infNFe.Id.valor)[3:].zfill(44)[x:x + y] for x, y in zip(
                    range(0, 44, 4), [int(x) for x in '44444444444'])])
            self.auto_expand_height = True


class Change(BandDANFE):
    """Troco"""
    def __init__(self):
        super(Change, self).__init__()
        self.add_label('Troco', top=0, left=0, width=13.8, style=ITEM)
        self.add_field('NFe.infNFe.pag.vTroco.valor', top=0, left=0, width=13.8, style=ITEM_RIGHT)


class LayoutLeft(BandDANFE):
    """Leiaute vertical do DANFE/NFC-e"""
    def __init__(self, qr_code=u'', contingency=False, cancel=False):
        super(LayoutLeft, self).__init__()
        # self.borders = dict(all=RoundRect(stroke_color=FUCHSIA))
        self.visible = False
        self.height = 3.3*cm
        fld = self.add_field('NFe.infNFe.dest.CPF.valor', top=.36, left=3, width=10.5, style=ITEM)
        fld.get_value = lambda i: u'<strong>CONSUMIDOR' + (
                i.NFe.infNFe.dest.CPF.valor and u' CPF: {}.{}.{}-{}'.format(
            *[str(i.NFe.infNFe.dest.CPF.valor).zfill(11)[x:x + y] for x, y in zip(
                range(0, 10, 3), (3, 3, 3, 2))]) or i.NFe.infNFe.dest.CNPJ.valor and
                u' CNPJ: {}.{}.{}/{}-{}'.format(
                    *[str(i.NFe.infNFe.dest.CNPJ.valor).zfill(14)[x:y] for x, y in zip(
                        (0, 2, 5, 8, 12), (2, 5, 8, 12, 14))])
                or u' NÃO IDENTIFICADO') + u'</strong>  ' + (
                i.NFe.infNFe.dest.xNome.valor and u', '.join([
            i.NFe.infNFe.dest.xNome.valor, i.NFe.infNFe.dest.enderDest.xLgr.valor,
            i.NFe.infNFe.dest.enderDest.nro.valor, i.NFe.infNFe.dest.enderDest.xBairro.valor,
            '-'.join([i.NFe.infNFe.dest.enderDest.xMun.valor,
                      i.NFe.infNFe.dest.enderDest.UF.valor])]))
        fld = self.add_field('NFe.infNFe.Id.valor', top=.36 * 3, left=3, width=10.5,
                             style=DEFAULT_BOLD)
        fld.get_value = lambda i: u'NFC-e n° {} Série {} {}'.format(
            i.NFe.infNFe.Id.valor[28:37], i.NFe.infNFe.Id.valor[25:28],
            i.protNFe.infProt.dhRecbto.valor or u'')
        fld = self.add_field('protNFe.infProt.nProt.valor', top=.36 * 4, left=3, width=10.5)
        fld.get_value = lambda i: i.protNFe.infProt.nProt.valor and \
                                  u'<strong>Protocolo de autorização:</strong> {} {} {}'.format(
                                      *[str(i.protNFe.infProt.nProt.valor).zfill(15)[x:x + y]
                                        for x, y in zip((0, 3, 13), (3, 10, 2))]) or u''
        self.add_label('Via consumidor', top=.36 * 5, left=3, width=10.5, style=DEFAULT_BOLD)

        # NFC-e cancelada
        top = 2.6
        if contingency:
            top = 3.0
        fld = self.add_label(
            u'NFC-e cancelada pelo emitente - Protocolo de cancelamento: %s' % cancel, top=top,
            left=3, width=10.5, style=OCCURRENCE_6B)
        fld.visible = bool(cancel)  # Mostra somente se cancelada

        # Contingencia
        fld = self.add_field('protNFe.infProt.dhRecbto.valor', top=.36 * 6, left=3, width=10.5)
        fld.get_value = lambda i:  u'<strong>Data de autorização:</strong> %s' % (
                i.protNFe.infProt.dhRecbto.valor and i.protNFe.infProt.dhRecbto.valor.strftime(
            '%d/%m/%Y %X') or u'Pendente')
        fld.visible = not contingency
        fld = self.add_label(u'EMITIDA EM CONTINGÊNCIA', top=2, left=2.95, width=10.5,
                             style=OCCURRENCE_13)
        fld.visible = contingency
        fld = self.add_label(u'Pendente de autorização', top=2.51, left=2.95, width=10.5,
                             style=OCCURRENCE)
        fld.visible = contingency

        # QR Code
        self.add_qrcode(qr_code, top=.2, left=.1, width=5, height=5)


class LayoutCenter(BandDANFE):
    def __init__(self, qr_code=u'', contingency=False, cancel=False):
        super(LayoutCenter, self).__init__()
        self.visible = False
        self.height = 7.4*cm
        fld = self.add_field('NFe.infNFe.dest.CPF.valor', top=0, left=0, width=13.8,
                             style=DEFAULT_CENTER)
        fld.get_value = lambda i: u'<strong>CONSUMIDOR</strong> CPF {}.{}.{}-{}'.format(
            *[str(i.NFe.infNFe.dest.CPF.valor).zfill(11)[x:x + y] for x, y in zip(
                range(0, 10, 3), (3, 3, 3, 2))]) + ' - ' + ', '.join(
            [i.NFe.infNFe.dest.enderDest.xLgr.valor, i.NFe.infNFe.dest.enderDest.nro.valor,
             i.NFe.infNFe.dest.enderDest.xBairro.valor, '-'.join([
                i.NFe.infNFe.dest.enderDest.xMun.valor, i.NFe.infNFe.dest.enderDest.UF.valor])])
        fld = self.add_field('protNFe.infProt.chNFe.valor', top=.36, left=0, width=13.8,
                             style=DEFAULT_BOLD_CENTER)
        fld.get_value = lambda i: u'NFC-e n° {} Série {} {} - Via consumidor'.format(
            i.protNFe.infProt.chNFe.valor[25:34], i.protNFe.infProt.chNFe.valor[22:25],
            i.protNFe.infProt.dhRecbto.valor)
        fld = self.add_field('protNFe.infProt.nProt.valor', top=.36 * 2, left=0, width=13.8,
                             style=DEFAULT_CENTER)
        fld.get_value = lambda i: u'<strong>Protocolo de autorização:</strong> {} {} {}'.format(
            *[str(i.protNFe.infProt.nProt.valor).zfill(15)[x:x + y] for x, y in zip(
                (0, 3, 13), (3, 10, 2))])
        fld = self.add_field('protNFe.infProt.dhRecbto', top=.36 * 3, left=0, width=13.8,
                             style=DEFAULT_CENTER)
        fld.get_value = lambda i:  u'<strong>Data de autorização:</strong> %s' % (
            i.protNFe.infProt.dhRecbto.valor.strftime('%d/%m/%Y %X')) \
            if not contingency else \
            u'<font size=13 color="#FF9393"><strong>EMITIDA EM CONTINGÊNCIA</strong></font><br/>' \
            u'<font color="#FF9393"><strong>Pendente de autorizacao</strong></font>'

        # QR Code
        self.add_qrcode(qr_code, top=.36 * 4, left=4.1, width=10, height=10)

class DANFENfceSimples(DANFENfce):
    def __init__(self, queryset=None):
        super(DANFENfceSimples, self).__init__(queryset, logo=False)
        self._logo = False
        self.items.band_header.visible = False
        self.items.band_detail.visible = False
        self.items.band_footer.elements = filter(
            lambda f: not isinstance(f, SubReport), self.items.band_footer.elements)
        

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
