# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.nfe_400.manual_600.nfe_400 import XMLNFe, TagDecimal, TagCaracter, TagInteiro, Signature
from nfe.nfe_400.manual_600.nfe_400 import NAMESPACE_NFE


class RetInfEvento(XMLNFe):
    """Grupo de informacoes do registro do Evento"""
    def __init__(self):
        super(RetInfEvento, self).__init__()
        self.Id = TagCaracter(
            nome=u'infEvento', codigo=u'HR12', propriedade=u'Id', tamanho=[17, 17],
            raiz=u'//retEvento')
        self.tpAmb = TagInteiro(nome=u'tpAmb', codigo=u'HR13', tamanho=[1, 1], raiz=u'//infEvento')
        self.verAplic = TagCaracter(
            nome=u'verAplic', codigo=u'HR14', tamanho=[1, 20], raiz=u'//infEvento')
        self.cOrgao = TagCaracter(
            nome=u'cOrgao', codigo=u'H1R5', tamanho=[2, 2], raiz=u'//infEvento')
        self.cStat = TagInteiro(nome=u'cStat', codigo=u'HR16', tamanho=[3, 3], raiz=u'//infEvento')
        self.xMotivo = TagCaracter(
            nome=u'xMotivo', codigo=u'HR17', tamanho=[1, 255], raiz=u'//infEvento')
        self.chNFe = TagCaracter(
            nome=u'chNFe', codigo=u'HR18', tamanho=[44, 44], raiz=u'//infEvento')
        self.tpEvento = TagCaracter(
            nome=u'tpEvento', codigo=u'HR19', tamanho=[6, 6], raiz=u'//infEvento',
            valor=u'110111')
        self.xEvento = TagCaracter(
            nome=u'xEvento', codigo=u'HR20', tamanho=[5, 60], raiz=u'//infEvento',
            valor=u'CANCELAMENTO')
        self.dhRegEvento = TagCaracter(
            nome=u'dhRegEvento', codigo=u'HR25', tamanho=[0, 30], raiz=u'//infEvento')

    @property
    def xml(self):
        xml = self.Id.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.chNFe.xml
        xml += self.tpEvento.xml
        xml += self.xEvento.xml
        xml += self.dhRegEvento.xml
        xml += u'</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cOrgao.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.chNFe.xml = arquivo
            self.tpEvento.xml = arquivo
            self.xEvento.xml = arquivo
            self.dhRegEvento.xml = arquivo


class RetEvento(XMLNFe):
    """Grupo do resultado do processamento do Evento (cancelamento)"""
    def __init__(self):
        super(RetEvento, self).__init__()
        self.versao = TagDecimal(
            nome=u'retEvento', codigo=u'HR09', propriedade=u'versao', valor=u'1.00',
            namespace=NAMESPACE_NFE, raiz=u'/')
        self.infEvento = RetInfEvento()

    @property
    def xml(self):
        xml = self.versao.xml
        xml += self.infEvento.xml
        xml += u'</retEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo


class EventoInfEvento(XMLNFe):
    """Grupo de informacoes do evento de cancelamento"""
    def __init__(self):
        super(EventoInfEvento, self).__init__()
        self.Id = TagCaracter(
            nome=u'infEvento', codigo=u'', propriedade=u'Id', tamanho=[54, 54], raiz=u'/')

    @property
    def xml(self):
        xml = self.Id.xml
        xml += u'</infEvento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo


class Evento(XMLNFe):
    """Grupo de evento"""
    def __init__(self):
        super(Evento, self).__init__()
        self.versao = TagDecimal(
            nome=u'evento', codigo=u'', propriedade=u'versao', valor=u'1.00',
            namespace=NAMESPACE_NFE, raiz=u'/')
        self.infEvento = EventoInfEvento()
        self.Signature = Signature()

    @property
    def xml(self):
        xml = self.versao.xml
        xml += self.infEvento.xml
        xml += self.Signature.xml
        xml += u'</evento>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo
            self.Signature.xml = arquivo


class ProcEventoNFe(XMLNFe):
    def __init__(self):
        super(ProcEventoNFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'procEventoNFe', codigo=u'', propriedade=u'versao', valor=u'1.00',
            namespace=NAMESPACE_NFE, raiz=u'/')
        self.evento = Evento()
        self.retEvento = RetEvento()

    @property
    def xml(self):
        xml = self.versao.xml
        xml += self.evento.xml
        xml += self.retEvento.xml
        xml += u'</procEventoNFe'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.evento.xml = arquivo
            self.retEvento.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
