# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
import os
from nfe.pysped.xml_sped import *
from nfe.pysped.nfe.manual_500 import conssitnfe_310
from nfe.nfe_400.manual_600.nfe_400 import ESQUEMA_ATUAL
from nfe.nfe_400.manual_600.consrecinfe import ProtNFe
from nfe.nfe_400.manual_600.cancnfe import RetCancNFe

DIRNAME = os.path.dirname(__file__)


class ConsSitNFe(conssitnfe_310.ConsSitNFe):
    def __init__(self):
        super(ConsSitNFe, self).__init__()
        self.versao = TagDecimal(nome=u'consSitNFe', codigo=u'EP01', propriedade=u'versao',
                                 namespace=NAMESPACE_NFE, valor=u'4.00', raiz=u'/')
        self.caminho_esquema = os.path.join(DIRNAME, u'schema', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'consSitNFe_v4.00.xsd'


class DetEvento(XMLNFe):
    def __init__(self):
        super(DetEvento, self).__init__()
        self.versao = TagDecimal(
            nome=u'detEvento', codigo=u'', propriedade=u'versao', namespace=NAMESPACE_NFE,
            valor=u'1.00', raiz=u'/')
        self.descEvento = TagCaracter(
            nome=u'descEvento', codigo=u'', tamanho=[5, 60], valor=u'Cancelamento',
            raiz=u'//detEvento')
        self.nProt = TagInteiro(nome=u'nProt', codigo=u'', tamanho=[15, 15], raiz=u'//detEvento')
        self.xJust = TagCaracter(nome=u'xJust', codigo=u'', tamanho=[15, 255], raiz=u'//detEvento')

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.descEvento.xml
        xml += self.nProt.xml
        xml += self.xJust.xml
        xml += u'</detEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.descEvento.xml = arquivo
            self.nProt.xml = arquivo
            self.xJust.xml = arquivo


class InfEvento(XMLNFe):
    def __init__(self):
        super(InfEvento, self).__init__()
        self.Id = TagCaracter(nome=u'infEvento', codigo=u'', tamanho=[54, 54], raiz=u'/',
                              propriedade=u'Id')
        self.cOrgao = TagInteiro(nome=u'cOrgao', codigo=u'', tamanho=[2, 2, 2], raiz=u'//infEvento')
        self.tpAmb = TagInteiro(nome=u'tpAmb', codigo=u'', tamanho=[1, 1, 1], raiz=u'//infEvento')
        self.CNPJ = TagCaracter(nome=u'CNPJ', codigo=u'', tamanho=[0, 14], raiz=u'//infEvento',
                                obrigatorio=False)
        self.CPF = TagCaracter(nome=u'CPF', codigo=u'', tamanho=[11, 11], raiz=u'//infEvento',
                               obrigatorio=False)
        self.chNFe = TagCaracter(nome=u'chNFe', codigo=u'', tamanho=[44, 44, 44],
                                 raiz=u'//infEvento', obrigatorio=False)
        self.dhEvento = TagCaracter(nome=u'dhEvento', codigo=u'' ,  tamanho=[0, 30],
                                    raiz=u'//infEvento')
        self.tpEvento = TagCaracter(nome=u'tpEvento', codigo=u'', tamanho=[6, 6, 6],
                                    raiz=u'//infEvento', valor=u'110111', obrigatorio=False)
        self.nSeqEvento = TagInteiro(nome=u'nSeqEvento', codigo=u'', tamanho=[2,2],
                                     raiz=u'//infEvento', obrigatorio=False, valor=1)
        self.verEvento = TagCaracter(nome=u'verEvento', codigo=u'' , tamanho=[1, 20],
                                     raiz=u'//infEvento')
        self.detEvento = DetEvento()

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.Id.xml
        xml += self.cOrgao.xml
        xml += self.tpAmb.xml
        if self.CPF.xml:
            xml += self.CPF.xml
        else:
            xml += self.CNPJ.xml
        xml += self.chNFe.xml
        xml += self.dhEvento.xml
        xml += self.tpEvento.xml
        xml += self.nSeqEvento.xml
        xml += self.verEvento.xml
        xml += u'</infEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.cOrgao.xml = arquivo
            self.tpAmb.xml = arquivo
            self.CPF.xml = arquivo
            self.CNPJ.xml = arquivo
            self.chNFe.xml = arquivo
            self.dhEvento.xml = arquivo
            self.tpEvento.xml = arquivo
            self.nSeqEvento.xml = arquivo
            self.verEvento.xml = arquivo


class Evento(XMLNFe):
    def __init__(self):
        super(Evento, self).__init__()
        self.versao = TagDecimal(nome=u'evento', codigo=u'', propriedade=u'versao',
                                 valor=u'1.00', raiz=u'/', namespace=NAMESPACE_NFE)
        self.infEvento = InfEvento()
        self.signature = Signature()

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.infEvento.xml
        xml += self.signature.xml
        xml += u'</evento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo
            self.signature.xml = arquivo


class RetInfEvento(XMLNFe):
    def __init__(self):
        super(RetInfEvento, self).__init__()
        self.tpAmb = TagInteiro(
            nome=u'tpAmb', codigo=u'', tamanho=[1, 1, 1], raiz=u'//retEvento/infEvento')
        self.verAplic = TagCaracter(
            nome=u'verAplic', codigo=u'', tamanho=[1, 20], raiz=u'//retEvento/infEvento')
        self.cOrgao = TagInteiro(
            nome=u'cOrgao', codigo=u'', tamanho=[2, 2, 2], raiz=u'//retEvento/infEvento')
        self.cStat = TagCaracter(
            nome=u'cStat', codigo=u'', tamanho=[3, 3, 3], raiz=u'//retEvento/infEvento')
        self.xMotivo = TagCaracter(
            nome=u'xMotivo', codigo=u'', tamanho=[1, 255], raiz=u'//retEvento/infEvento')
        self.chNFe = TagCaracter(
            nome=u'chNFe', codigo=u'', tamanho=[44, 44, 44], raiz=u'//retEvento/infEvento',
            obrigatorio=False)
        self.tpEvento = TagCaracter(
            nome=u'tpEvento', codigo=u'', tamanho=[6, 6, 6], raiz=u'//retEvento/infEvento',
            obrigatorio=False)
        self.xEvento = TagCaracter(
            nome=u'xEvento', codigo=u'', tamanho=[5, 60], raiz=u'//retEvento/infEvento',
            obrigatorio=False)
        self.nSeqEvento = TagInteiro(
            nome=u'nSeqEvento', codigo=u'', tamanho=[1, 2, 1],  raiz=u'//retEvento/infEvento',
            valor=1, obrigatorio=False)
        self.CNPJDest = TagCaracter(
            nome=u'CNPJDest', codigo=u'', tamanho=[0, 14], raiz=u'//retEvento/infEvento',
            obrigatorio=False)
        self.CPFDest = TagCaracter(
            nome=u'CPFDest', codigo=u'', tamanho=[11, 11], raiz=u'//retEvento/infEvento',
            obrigatorio=False)
        self.emailDest = TagCaracter(
            nome=u'emailDest', codigo=u'', tamanho=[1, 60], raiz=u'//retEvento/infEvento',
            obrigatorio=False)
        self.dhRegEvento = TagCaracter(
            nome=u'dhRegEvento', codigo=u'', tamanho=[0, 30], raiz=u'//retEvento/infEvento')
        self.nProt = TagCaracter(
            nome=u'nProt', codigo=u'', tamanho=[15, 15, 15], raiz=u'//retEvento/infEvento',
            obrigatorio=False)

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<infEvento>'
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cOrgao.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.chNFe.xml
        xml += self.tpEvento.xml
        xml += self.xEvento.xml
        xml += self.nSeqEvento.xml
        if self.CPFDest.xml:
            xml += self.CPFDest.xml
        else:
            xml += self.CNPJDest.xml
        xml += self.emailDest.xml
        xml += self.dhRegEvento.xml
        xml += self.nProt.xml
        xml += u'</infEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cOrgao.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.chNFe.xml = arquivo
            self.tpEvento.xml = arquivo
            self.xEvento.xml = arquivo
            self.nSeqEvento.xml = arquivo
            self.CNPJDest.xml = arquivo
            self.CPFDest.xml = arquivo
            self.emailDest.xml = arquivo
            self.dhRegEvento.xml = arquivo
            self.nProt.xml = arquivo


class RetEvento(XMLNFe):
    def __init__(self):
        super(RetEvento, self).__init__()
        self.versao = TagDecimal(nome=u'retEvento', codigo=u'', propriedade=u'versao',
                                 valor=u'1.00', raiz=u'/', namespace=NAMESPACE_NFE)
        self.infEvento = RetInfEvento()

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.infEvento.xml
        xml += u'</retEvento>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.infEvento.xml = arquivo


class ProcEventoNFe(XMLNFe):
    def __init__(self):
        super(ProcEventoNFe, self).__init__()
        self.versao = TagDecimal(nome=u'procEventoNFe', codigo=u'', propriedade=u'versao',
                                 valor=u'1.00', raiz=u'/', namespace=NAMESPACE_NFE)
        self.evento = Evento()
        self.retEvento = RetEvento()

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.evento.xml
        xml += self.retEvento.xml
        xml += u'</procEventoNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        self.versao.xml = arquivo
        self.evento.xml = arquivo
        self.retEvento.xml = arquivo


class RetConsSitNFe(conssitnfe_310.RetConsSitNFe):
    def __init__(self):
        super(RetConsSitNFe, self).__init__()
        self.versao = TagDecimal(nome=u'retConsSitNFe', codigo=u'ER01', propriedade=u'versao',
                                 namespace=NAMESPACE_NFE, valor=u'4.00', raiz=u'/')
        self.dhRecbto = TagDataHoraUTC(nome=u'dhRecbto', codigo=u'ER07a', raiz=u'/')
        self.procEventoNFe = []
        self.caminho_esquema = os.path.join(DIRNAME, u'schema', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'retConsSitNFe_v4.00.xsd'

    @property
    def xml(self):
        xml = ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.chNFe.xml
        if self.protNFe is not None:
            xml += self.protNFe.xml
        if self.retCancNFe is not None:
            xml += tira_abertura(self.retCancNFe.xml)
        for p in self.procEventoNFe:
            xml += p.xml
        xml += u'</retConsSitNFe>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpAmb.xml = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml = arquivo
            self.xMotivo.xml = arquivo
            self.cUF.xml = arquivo
            self.dhRecbto.xml = arquivo
            self.chNFe.xml = arquivo
            if self._le_noh(u'//retConsSitNFe/protNFe'):
                self.protNFe = ProtNFe()
                self.protNFe.xml = arquivo
            if self._le_noh(u'//retConsSitNFe/retCancNFe'):
                self.retCancNFe = RetCancNFe()
                self.retCancNFe.xml = arquivo

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
