# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
import os
from nfe.pysped.xml_sped import *
from nfe.pysped.nfe.manual_500 import nfe_310

DIRNAME = os.path.dirname(__file__)
ESQUEMA_ATUAL = u'PL_009k_NT2023_001_v120'

class Lacres(nfe_310.Lacres):
    def __init__(self):
        super(Lacres, self).__init__()


class Vol(nfe_310.Vol):
    def __init__(self, xml=None):
        super(Vol, self).__init__()

    @property
    def xml(self):
        if not (self.qVol.valor or self.nVol.valor or self.pesoL.valor or self.pesoB.valor):
            return u''
        xml = u'<vol>'
        xml += self.qVol.xml
        xml += self.esp.xml
        xml += self.marca.xml
        xml += self.nVol.xml
        xml += self.pesoL.xml
        xml += self.pesoB.xml
        for l in self.lacres:
            xml += l.xml
        xml += u'</vol>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.qVol.xml = arquivo
            self.esp.xml = arquivo
            self.marca.xml = arquivo
            self.nVol.xml = arquivo
            self.pesoL.xml = arquivo
            self.pesoB.xml = arquivo
            self.lacres = self.le_grupo(u'//vol/lacres', Lacres)


class II(nfe_310.II):
    def __init__(self):
        super(II, self).__init__()


class IPI(nfe_310.IPI):
    def __init__(self):
        super(IPI, self).__init__()
        del self.clEnq

    @property
    def xml(self):
        if not ((self.CST.valor in (u'00', u'49', u'50', u'99')) or
                    (self.qUnid.valor or self.vUnid.valor or self.vBC.valor or self.pIPI.valor or
                         self.vIPI.valor)):
            return u''
        xml = u'<IPI>'
        xml += self.CNPJProd.xml
        xml += self.cSelo.xml
        xml += self.qSelo.xml
        xml += self.cEnq.xml
        xml += u'<' + self.nome_tag + u'>'
        xml += self.CST.xml
        if self.CST.valor in (u'00', u'49', u'50', u'99'):
            if self.qUnid.valor or self.vUnid.valor:
                xml += self.qUnid.xml
                xml += self.vUnid.xml
            else:
                xml += self.vBC.xml
                xml += self.pIPI.xml
            xml += self.vIPI.xml
        xml += u'</' + self.nome_tag + u'></IPI>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            if self._le_noh(u'//det/imposto/IPI/IPINT'):
                self.CST.valor = u'01'
            else:
                self.CST.valor = u'00'
            self.CST.xml = arquivo
            self.CNPJProd.xml = arquivo
            self.cSelo.xml = arquivo
            self.qSelo.xml = arquivo
            self.cEnq.xml = arquivo
            self.vBC.xml = arquivo
            self.qUnid.xml = arquivo
            self.vUnid.xml = arquivo
            self.pIPI.xml = arquivo
            self.vIPI.xml = arquivo


class TagCSTICMS(nfe_310.TagCSTICMS):
    def __init__(self, *args, **kwargs):
        super(TagCSTICMS, self).__init__(*args, **kwargs)

    @property
    def valor(self):
        return super(TagCSTICMS, self).get_valor()

    @valor.setter
    def valor(self, novo_valor):
        super(TagCSTICMS, self).set_valor(novo_valor)
        if self.grupo_icms:
            # Definimos todas as tags como nao obrigatorias
            self.grupo_icms.vBCFCP.obrigatorio = False
            self.grupo_icms.pFCP.obrigatorio = False
            self.grupo_icms.vFCP.obrigatorio = False
            self.grupo_icms.vBCFCPST.obrigatorio = False
            self.grupo_icms.pFCPST.obrigatorio = False
            self.grupo_icms.vFCPST.obrigatorio = False
            self.grupo_icms.pST.obrigatorio = False
            self.grupo_icms.vICMSSubstituto.obrigatorio = False
            self.grupo_icms.vBCFCPSTRet.obrigatorio = False
            self.grupo_icms.pFCPSTRet.obrigatorio = False
            self.grupo_icms.vFCPSTRet.obrigatorio = False
            self.grupo_icms.vICMSOp.obrigatorio = False
            self.grupo_icms.pDif.obrigatorio = False
            self.grupo_icms.vICMSDif.obrigatorio = False

            # Por seguranca, zeramos os valores das tags do grupo ICMS ao redefinirmos o codigo da
            # situacao tributaria
            self.grupo_icms.vBCFCP.valor = u'0.00'
            self.grupo_icms.pFCP.valor = u'0.00'
            self.grupo_icms.vFCP.valor = u'0.00'
            self.grupo_icms.vBCFCPST.valor = u'0.00'
            self.grupo_icms.pFCPST.valor = u'0.00'
            self.grupo_icms.vFCPST.valor = u'0.00'
            self.grupo_icms.pST.valor = u'0.00'
            self.grupo_icms.vICMSSubstituto.valor = u'0.00'
            self.grupo_icms.vBCFCPSTRet.valor = u'0.00'
            self.grupo_icms.pFCPSTRet.valor = u'0.00'
            self.grupo_icms.vFCPSTRet.valor = u'0.00'
            self.grupo_icms.vICMSOp.valor = u'0.00'
            self.grupo_icms.pDif.valor = u'0.00'
            self.grupo_icms.vICMSDif.valor = u'0.00'

            # Para cada codigo de situacao tributaria, redefinimos a raiz e a obrigatoriedade das
            # tags do grupo de ICMS
            if self.valor == u'20':
                self.grupo_icms.vICMSDeson.obrigatorio = True
                self.grupo_icms.motDesICMS.obrigatorio = True
            elif self.valor == u'30':
                self.grupo_icms.vICMSDeson.obrigatorio = True
                self.grupo_icms.motDesICMS.obrigatorio = True
            elif self.valor in (u'40', u'41', u'50'):
                self.grupo_icms.vICMSDeson.obrigatorio = True
                self.grupo_icms.motDesICMS.obrigatorio = True
            elif self.valor == u'60':
                if self.grupo_icms.repasse:
                    self.grupo_icms.nome_tag = u'ICMSST'
                    self.grupo_icms.raiz_tag = u'//det/imposto/ICMS/ICMSST'
                    self.grupo_icms.vBCSTRet.obrigatorio = True
                    self.grupo_icms.vICMSSTRet.obrigatorio = True
                    self.grupo_icms.vBCSTDest.obrigatorio = True
                    self.grupo_icms.vICMSSTDest.obrigatorio = True
                self.grupo_icms.pST.obrigatorio = True
                self.grupo_icms.vICMSSubstituto.obrigatorio = True
            elif self.valor == u'61':
                self.grupo_icms.adRemICMSRet.obrigatorio = True
                self.grupo_icms.vICMSMonoRet.obrigatorio = True
            elif self.valor == u'70':
                self.grupo_icms.vICMSDeson.obrigatorio = True
                self.grupo_icms.motDesICMS.obrigatorio = True
            elif self.valor == u'90':
                self.grupo_icms.vICMSDeson.obrigatorio = True
                self.grupo_icms.motDesICMS.obrigatorio = True

            # Redefine a raiz para todas as tags do grupo ICMS
            self.grupo_icms.orig.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.motDesICMS.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSDeson.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCSTDest.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSTDest.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.UFST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pBCOp.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCFCP.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pFCP.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vFCP.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCFCPST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pFCPST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vFCPST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSubstituto.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCFCPSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pFCPSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vFCPSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSOp.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pDif.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSDif.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.qBCMonoRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.adRemICMSRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSMonoRet.raiz = self.grupo_icms.raiz_tag


class TagCSOSN(nfe_310.TagCSOSN):
    def __init__(self, *args, **kwargs):
        super(TagCSOSN, self).__init__(*args, **kwargs)

    @property
    def valor(self):
        return super(TagCSOSN, self).get_valor()

    @valor.setter
    def valor(self, novo_valor):
        super(TagCSOSN, self).set_valor(novo_valor)
        if self.grupo_icms:
            # Para cada codigo de situacao tributaria, redefinimos a obrigatoriedade das tags do
            # grupo de ICMS
            if self.valor == u'500':
                self.grupo_icms.pST.obrigatorio = True
                self.grupo_icms.vICMSSubstituto.obrigatorio = True
                self.grupo_icms.vICMSSTRet.obrigatorio = True
                self.grupo_icms.vBCFCPSTRet.obrigatorio = False
                self.grupo_icms.pFCPSTRet.obrigatorio = False
                self.grupo_icms.vFCPSTRet.obrigatorio = False

                # Por seguranca, zeramos os valores das tags do grupo ICMS ao redefinirmos o codigo
                # da situacao tributaria
                self.grupo_icms.pST.valor = u'0.00'
                self.grupo_icms.vICMSSubstituto.valor = u'0.00'
                self.grupo_icms.vICMSSTRet.valor = u'0.00'
                self.grupo_icms.vBCFCPSTRet.valor = u'0.00'
                self.grupo_icms.pFCPSTRet.valor = u'0.00'
                self.grupo_icms.vFCPSTRet.valor = u'0.00'

            # Redefine a raiz para todas as tags do grupo ICMS
            self.grupo_icms.pST.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSSubstituto.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCFCPSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pFCPSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vFCPSTRet.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vBCFCP.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pFCP.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vFCP.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSOp.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.pDif.raiz = self.grupo_icms.raiz_tag
            self.grupo_icms.vICMSDif.raiz = self.grupo_icms.raiz_tag


class ICMS(nfe_310.ICMS):
    def __init__(self):
        super(ICMS, self).__init__()
        self.vICMSOp = TagDecimal(
            nome=u'vICMSOp', codigo=u'N16a', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.pDif = TagDecimal(nome=u'pDif', codigo=u'N16b', tamanho=[1, 7, 1], decimais=[0, 4, 4])
        self.vICMSDif = TagDecimal(
            nome=u'vICMSDif', codigo=u'N16c', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.vBCFCP = TagDecimal(
            nome=u'vBCFCP', codigo=u'N17a', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.pFCP = TagDecimal(nome=u'pFCP', codigo=u'N17b', tamanho=[1,  7, 1], decimais=[0, 4, 2])
        self.vFCP = TagDecimal(nome=u'vFCP', codigo=u'N17c', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.vBCFCPST = TagDecimal(
            nome=u'vBCFCPST', codigo=u'N23a', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.pFCPST = TagDecimal(
            nome=u'pFCPST', codigo=u'N23b', tamanho=[1,  7, 1], decimais=[0, 4, 4])
        self.vFCPST = TagDecimal(
            nome=u'vFCPST', codigo=u'N23c', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.pST = TagDecimal(nome=u'pST', codigo=u'N26a', tamanho=[1, 7, 1], decimais=[0, 4, 4])
        self.vICMSSubstituto = TagDecimal(
            nome=u'vICMSSubstituto', codigo=u'N26b', tamanho=[1, 15, 1], decimais=[0, 2, 2])
        self.vBCFCPSTRet = TagDecimal(nome=u'vBCFCPSTRet', codigo=u'N27a', tamanho=[1, 15, 1],
                                      decimais=[0, 2, 2])
        self.pFCPSTRet = TagDecimal(nome=u'pFCPSTRet', codigo=u'N27b', tamanho=[1, 7, 1],
                                    decimais=[0, 4, 4])
        self.vFCPSTRet = TagDecimal(nome=u'vFCPSTRet', codigo=u'N27d', tamanho=[1, 15, 1],
                                    decimais=[0, 2, 2])
        self.qBCMonoRet = TagDecimal(nome=u'qBCMonoRet', codigo=u'N43a', tamanho=[1, 15, 1],
                                  decimais=[0, 4, 4], raiz='', obrigatorio=True)
        self.adRemICMSRet = TagDecimal(nome=u'adRemICMSRet', codigo=u'N44', tamanho=[1, 7, 1],
                                  decimais=[0, 4, 2], raiz='', obrigatorio=True)
        self.vICMSMonoRet = TagDecimal(nome=u'vICMSMonoRet', codigo=u'N45', tamanho=[1, 15, 1],
                                  decimais=[0, 2, 2], raiz='', obrigatorio=True)
        self.CST = TagCSTICMS()
        self.CST.grupo_icms = self
        self.CSOSN = TagCSOSN()
        self.CSOSN.grupo_icms = self

    @property
    def xml(self):
        xml = u'<ICMS><' + self.nome_tag + u'>'
        xml += self.orig.xml
        if self.regime_tributario != 1:
            xml += self.CST.xml
            if self.CST.valor == u'00':
                xml += self.modBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                if self.vFCP.valor:
                    xml += self.pFCP.xml
                    xml += self.vFCP.xml
            elif self.CST.valor == u'10':  # Vale para ICMSPart - Partilha
                xml += self.modBC.xml
                xml += self.vBC.xml
                if self.partilha:
                    xml += self.pRedBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.vFCPST.valor:
                    xml += self.vBCFCP.xml
                    xml += self.pFCPST.xml
                    xml += self.vFCPST.xml
            elif self.CST.valor == u'20':
                xml += self.modBC.xml
                xml += self.pRedBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                if self.vFCP.valor:
                    xml += self.vBCFCP.xml
                    xml += self.pFCP.xml
                    xml += self.vFCP.xml
                if self.vICMSDeson.valor:
                    xml += self.vICMSDeson.xml
                    xml += self.motDesICMS.xml
            elif self.CST.valor == u'30':
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.vFCPST.valor:
                    xml += self.vBCFCPST.xml
                    xml += self.pFCPST.xml
                    xml += self.vFCPST.xml
                if self.vICMSDeson.valor:
                    xml += self.vICMSDeson.xml
                    xml += self.motDesICMS.xml
            elif self.CST.valor in (u'40', u'41', u'50'):
                if self.repasse and self.CST.valor == u'41':
                    xml += self.vBCSTRet.xml
                    xml += self.vICMSSTRet.xml
                    xml += self.vBCSTDest.xml
                    xml += self.vICMSSTDest.xml
                else:
                    if self.vICMSDeson.valor:
                        xml += self.vICMSDeson.xml
                        xml += self.motDesICMS.xml
            elif self.CST.valor == u'51':
                xml += self.modBC.xml
                xml += self.pRedBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMSOp.xml
                xml += self.pDif.xml
                xml += self.vICMSDif.xml
                xml += self.vICMS.xml
                if self.vFCP.valor:
                    xml += self.vBCFCP.xml
                    xml += self.pFCP.xml
                    xml += self.vFCP.xml
            elif self.CST.valor == u'60':
                xml += self.vBCSTRet.xml
                xml += self.pST.xml
                if self.repasse:
                    xml += self.vICMSSubstituto.xml
                    xml += self.vICMSSTRet.xml
                    xml += self.vBCSTDest.xml
                    xml += self.vICMSSTDest.xml
                else:
                    xml += self.vICMSSubstituto.xml
                    xml += self.vICMSSTRet.xml
                    xml += self.vBCFCPSTRet.xml
                    xml += self.pFCPSTRet.xml
                    xml += self.vFCPSTRet.xml
            elif self.CST.valor == u'61':
                xml += self.qBCMonoRet.xml
                xml += self.adRemICMSRet.xml
                xml += self.vICMSMonoRet.xml
            elif self.CST.valor == u'70':
                xml += self.modBC.xml
                xml += self.pRedBC.xml
                xml += self.vBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                if self.vFCP.valor:
                    xml += self.vBCFCP.xml
                    xml += self.pFCP.xml
                    xml += self.vFCP.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.vFCPST.valor:
                    xml += self.vBCFCPST.xml
                    xml += self.pFCPST.xml
                    xml += self.vFCPST.xml
                if self.vICMSDeson.valor:
                    xml += self.vICMSDeson.xml
                    xml += self.motDesICMS.xml
            elif self.CST.valor == u'90':
                xml += self.modBC.xml
                xml += self.vBC.xml
                if self.partilha:
                    xml += self.pRedBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                if not self.partilha and self.vFCP.valor:
                    xml += self.vBCFCP.xml
                    xml += self.pFCP.xml
                    xml += self.vFCP.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.partilha:
                    xml += self.pBCOp.xml
                    xml += self.UFST.xml
                else:
                    if self.vFCPST.valor:
                        xml += self.vBCFCPST.xml
                        xml += self.pFCPST.xml
                        xml += self.vFCPST.xml
                    if self.vICMSDeson.valor:
                        xml += self.vICMSDeson.xml
                        xml += self.motDesICMS.xml
        else:  # Simples Nacional
            xml += self.CSOSN.xml
            if self.CSOSN.valor == u'101':
                xml += self.pCredSN.xml
                xml += self.vCredICMSSN.xml
            elif self.CSOSN.valor in (u'102', u'103', u'300', u'400'):
                pass
            elif self.CSOSN.valor == u'201':
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.vFCPST.valor:
                    xml += self.vBCFCPST.xml
                    xml += self.pFCPST.xml
                    xml += self.vFCPST.xml
                xml += self.pCredSN.xml
                xml += self.vCredICMSSN.xml
            elif self.CSOSN.valor in (u'202', u'203'):
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.vFCPST.valor:
                    xml += self.vBCFCPST.xml
                    xml += self.pFCPST.xml
                    xml += self.vFCPST.xml
            elif self.CSOSN.valor == u'500':
                xml += self.vBCSTRet.xml
                xml += self.pST.xml
                xml += self.vICMSSubstituto.xml
                xml += self.vICMSSTRet.xml
                xml += self.vBCFCPSTRet.xml
                xml += self.pFCPSTRet.xml
                xml += self.vFCPSTRet.xml
            elif self.CSOSN.valor == u'900':
                xml += self.modBC.xml
                xml += self.vBC.xml
                xml += self.pRedBC.xml
                xml += self.pICMS.xml
                xml += self.vICMS.xml
                xml += self.modBCST.xml
                if self.modBCST.valor == 4:
                    xml += self.pMVAST.xml
                xml += self.pRedBCST.xml
                xml += self.vBCST.xml
                xml += self.pICMSST.xml
                xml += self.vICMSST.xml
                if self.vFCPST.valor:
                    xml += self.vBCFCPST.xml
                    xml += self.pFCPST.xml
                    xml += self.vFCPST.xml
                xml += self.pCredSN.xml
                xml += self.vCredICMSSN.xml
        xml += u'</' + self.nome_tag + u'></ICMS>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            super(ICMS, self).set_xml(arquivo)

            # Para ler corretamente o ICMS, primeiro temos que descobrir em qual grupo de situacao
            # tributaria ele esta
            self.partilha = False
            self.repasse  = False
            if self._le_noh(u'//det/imposto/ICMS/ICMSST') is not None:
                self.regime_tributario = 3
                self.repasse = True
                self.CST.valor = self._le_noh(u'//det/imposto/ICMS/ICMSST/CST').text or u''

            self.orig.xml = arquivo
            self.vBCSTRet.xml = arquivo
            self.vICMSSTRet.xml = arquivo
            self.vBCSTDest.xml = arquivo
            self.vICMSSTDest.xml = arquivo
            self.vBCFCPST.xml = arquivo
            self.pFCPST.xml = arquivo
            self.vFCPST.xml = arquivo
            self.pST.xml = arquivo
            self.vICMSSubstituto.xml = arquivo
            self.vBCFCPSTRet.xml = arquivo
            self.pFCPSTRet.xml = arquivo
            self.vFCPSTRet.xml = arquivo
            self.vBCFCP.xml = arquivo
            self.pFCP.xml = arquivo
            self.vFCP.xml = arquivo
            self.vICMSDeson.xml = arquivo
            self.vICMSOp.xml = arquivo
            self.pDif.xml = arquivo
            self.vICMSDif.xml = arquivo
            self.qBCMonoRet.xml = arquivo
            self.adRemICMSRet.xml = arquivo
            self.vICMSMonoRet.xml = arquivo


class Imposto(nfe_310.Imposto):
    def __init__(self):
        super(Imposto, self).__init__()
        self.ICMS = ICMS()
        self.IPI = IPI()
        self.II = II()


class IPIDevol(XMLNFe):
    """Informacao do IPI devolvido"""
    def __init__(self):
        super(IPIDevol, self).__init__()
        self.vIPIDevol = TagDecimal(
            nome=u'vIPIDevol', codigo=u'U61', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//impostoDevol/IPI')

    @property
    def xml(self):
        xml = u'<IPI>'
        xml += self.vIPIDevol.xml
        xml += u'</IPI>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.vIPIDevol.xml = arquivo


class ImpostoDevolvido(XMLNFe):
    """Informacao do imposto devolvido"""
    def __init__(self):
        super(ImpostoDevolvido, self).__init__()
        self.pDevol = TagDecimal(
            nome=u'pDevol', codigo=u'U51', tamanho=[1, 5, 1], decimais=[0, 2, 2],
            raiz=u'//impostoDevol')
        self.IPI = IPIDevol()

    @property
    def xml(self):
        if not self.pDevol.valor:
            return u''
        xml = u'<impostoDevol>'
        xml += self.pDevol.xml
        xml += self.IPI.xml
        xml += u'</impostoDevol>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.pDevol.xml = arquivo
            self.IPI.xml = arquivo


class Encerrante(XMLNFe):
    def __init__(self):
        super(Encerrante, self).__init__()
        self.nBico = TagInteiro(nome=u'nBico', codigo=u'LA12', tamanho=[1, 3],
                                raiz=u'//det/prod/comb/encerrante')
        self.nBomba = TagInteiro(nome=u'nBomba', codigo=u'LA13', tamanho=[1, 3],
                                 raiz=u'//det/prod/comb/encerrante', obrigatorio=False)
        self.nTanque = TagInteiro(nome=u'nTanque', codigo=u'LA14', tamanho=[1, 3],
                                  raiz=u'//det/prod/comb/encerrante')
        self.vEncIni = TagDecimal(nome=u'vEncIni', codigo=u'LA15', tamanho=[1, 15, 1],
                                  decimais=[0, 4, 4], raiz=u'//det/prod/comb/encerrante')
        self.vEncFin = TagDecimal(nome=u'vEncFin', codigo=u'LA16', tamanho=[1, 15, 1],
                                  decimais=[0, 4, 4], raiz=u'//det/prod/comb/encerrante')

    @property
    def xml(self):
        if not self.vEncIni.valor:
            return u''
        xml = u'<encerrante>'
        xml += self.nBico.xml
        xml += self.nBomba.xml
        xml += self.nTanque.xml
        xml += self.vEncIni.xml
        xml += self.vEncFin.xml
        xml += u'</encerrante>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nBico.xml = arquivo
            self.nBomba.xml = arquivo
            self.nTanque.xml = arquivo
            self.vEncIni.xml = arquivo
            self.vEncFin.xml = arquivo


class Comb(nfe_310.Comb):
    def __init__(self):
        super(Comb, self).__init__()
        del self.pMixGN
        self.descANP = TagCaracter(nome=u'descANP', codigo=u'LA3', tamanho=[2, 95],
                                   raiz=u'//det/prod/comb')
        self.pGLP = TagDecimal(nome=u'pGLP', codigo=u'LA3a', tamanho=[1, 7, 1], decimais=[0, 4, 4],
                               raiz=u'//det/prod/comb', obrigatorio=False)
        self.pGNn = TagDecimal(nome=u'pGNn', codigo=u'LA3b', tamanho=[1, 7, 1], decimais=[0, 4, 4],
                               raiz=u'//det/prod/comb', obrigatorio=False)
        self.pGNi = TagDecimal(nome=u'pGNi', codigo=u'LA3c', tamanho=[1, 7, 1], decimais=[0, 4, 4],
                               raiz=u'//det/prod/comb', obrigatorio=False)
        self.vPart = TagDecimal(nome=u'vPart', codigo=u'LA3d', tamanho=[1, 15, 1],
                                decimais=[0, 2, 2], raiz=u'//det/prod/comb', obrigatorio=False)
        self.encerrante = Encerrante()

    @property
    def xml(self):
        if not self.cProdANP.valor:
            return u''
        xml = u'<comb>'
        xml += self.cProdANP.xml
        xml += self.descANP.xml
        xml += self.pGLP.xml
        xml += self.pGNn.xml
        xml += self.pGNi.xml
        xml += self.vPart.xml
        xml += self.CODIF.xml
        xml += self.qTemp.xml
        xml += self.UFCons.xml
        xml += self.CIDE.xml
        xml += self.encerrante.xml
        xml += u'</comb>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cProdANP.xml = arquivo
            self.descANP.xml = arquivo
            self.pGLP.xml = arquivo
            self.pGNn.xml = arquivo
            self.pGNi.xml = arquivo
            self.vPart.xml = arquivo
            self.CODIF.xml = arquivo
            self.qTemp.xml = arquivo
            self.UFCons.xml = arquivo
            self.CIDE.xml = arquivo
            self.encerrante.xml = arquivo


class Med(nfe_310.Med):
    def __init__(self):
        super(Med, self).__init__()
        del self.nLote
        del self.qLote
        del self.dFab
        del self.dVal
        self.cProdANVISA = TagCaracter(nome=u'cProdANVISA', codigo=u'K01a', tamanho=[1, 13],
                                       raiz=u'//med')
        self.xMotivoIsencao = TagCaracter(nome=u'xMotivoIsencao', codigo=u'K01b', tamanho=[1, 255],
                                          raiz=u'//med')

    @property
    def xml(self):
        if not self.cProdANVISA.valor:
            return u''

        xml = u'<med>'
        xml += self.cProdANVISA.xml
        if self.xMotivoIsencao.valor:
            xml += self.xMotivoIsencao.xml
        xml += self.vPMC.xml
        xml += u'</med>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cProdANVISA.xml = arquivo
            self.xMotivoIsencao.xml = arquivo
            self.vPMC.xml = arquivo


class Rastro(XMLNFe):
    def __init__(self):
        super(Rastro, self).__init__()
        self.nLote = TagCaracter(nome=u'nLote', codigo=u'I81', tamanho=[1, 20],
                                 raiz=u'//det/prod/rastro', obrigatorio=True)
        self.qLote = TagDecimal(nome=u'qLote', codigo=u'I82', raiz=u'//det/prod/rastro',
                                tamanho=[1, 8, 1], decimais=[0, 3, 3], obrigatorio=True)
        self.dFab = TagData(nome=u'dFab', codigo=u'I83', raiz=u'//det/prod/rastro',
                            obrigatorio=True)
        self.dVal = TagData(nome=u'dVal', codigo=u'I84', raiz=u'//det/prod/rastro',
                            obrigatorio=True)
        self.cAgreg = TagCaracter(nome=u'cAgreg', codigo=u'I85', tamanho=[1, 20],
                                  raiz=u'//det/prod/rastro', obrigatorio=False)

    @property
    def xml(self):
        if not self.nLote.valor:
            return u''
        xml = u'<rastro>'
        xml += self.nLote.xml
        xml += self.qLote.xml
        xml += self.dFab.xml
        xml += self.dVal.xml
        if self.cAgreg.valor:
            xml += self.cAgreg.xml

        xml += u'</rastro>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nLote.xml = arquivo
            self.dFab.xml = arquivo
            self.dVal.xml = arquivo
            self.cAgreg.xml = arquivo


class DI(nfe_310.DI):
    def __init__(self):
        super(DI, self).__init__()


class ExportInd(XMLNFe):
    def __init__(self):
        super(ExportInd, self).__init__()
        self.nRE = TagInteiro(nome=u'nRE', codigo=u'I53', tamanho=[1, 12], raiz=u'//exportInd')
        self.chNFe = TagInteiro(nome=u'chNFe', codigo=u'I54', tamanho=[1, 44], raiz=u'//exportInd')
        self.qExport = TagDecimal(nome=u'qExport', codigo=u'I55', tamanho=[1, 15, 1],
                                  decimais=[0, 4, 4], raiz=u'//exportInd')

    @property
    def xml(self):
        if not self.nRE.valor:
            return u''
        xml = u'<exportInd>'
        xml += self.nRE.xml
        xml += self.chNFe.xml
        xml += self.qExport.xml
        xml += u'</exportInd>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nRE.xml = arquivo
            self.chNFe.xml = arquivo


class DetExport(XMLNFe):
    def __init__(self):
        super(DetExport, self).__init__()
        self.nDraw = TagInteiro(nome=u'nDraw', codigo=u'I51', tamanho=[0, 11], raiz=u'//detExport',
                                obrigatorio=False)
        self.exportInd = ExportInd()

    @property
    def xml(self):
        if not self.nDraw.valor:
            return u''
        xml = u'<detExport>'
        xml += self.nDraw.xml
        xml += self.exportInd.xml
        xml += u'</detExport>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nDraw.xml = arquivo
            self.exportInd.xml = arquivo


class Arma(nfe_310.Arma):
    def __init__(self):
        super(Arma, self).__init__()


class Prod(nfe_310.Prod):
    def __init__(self):
        super(Prod, self).__init__()
        self.indEscala = TagCaracter(nome=u'indEscala', codigo=u'I05d', tamanho=[1, 1],
                                     raiz=u'//det/prod', obrigatorio=False)
        self.CNPJFab = TagCaracter(nome=u'CNPJFab', codigo=u'I05e', tamanho=[14, 14],
                                   raiz=u'//det/prod', obrigatorio=False)
        self.cBenef = TagCaracter(nome=u'cBenef', codigo=u'I05f', tamanho=[0, 10],
                                  raiz=u'//det/prod', obrigatorio=False)
        self.rastro = []
        self.detExport = []
        self.comb = Comb()
        self.med = Med()

    @property
    def xml(self):
        xml = u'<prod>'
        xml += self.cProd.xml
        xml += self.cEAN.xml
        xml += self.xProd.xml
        xml += self.NCM.xml
        xml += self.NVE.xml
        xml += self.CEST.xml
        xml += self.indEscala.xml
        if self.indEscala.valor == u'N':
            xml += self.CNPJFab.xml
        xml += self.cBenef.xml
        xml += self.EXTIPI.xml
        xml += self.CFOP.xml
        xml += self.uCom.xml
        xml += self.qCom.xml
        xml += self.vUnCom.xml
        xml += self.vProd.xml
        xml += self.cEANTrib.xml
        xml += self.uTrib.xml
        xml += self.qTrib.xml
        xml += self.vUnTrib.xml
        xml += self.vFrete.xml
        xml += self.vSeg.xml
        xml += self.vDesc.xml
        xml += self.vOutro.xml
        xml += self.indTot.xml
        for d in self.DI:
            xml += d.xml
        for de in self.detExport:
            xml += de.xml
        xml += self.xPed.xml
        xml += self.nItemPed.xml
        xml += self.nFCI.xml
        for r in self.rastro:
            xml += r.xml
        xml += self.veicProd.xml
        xml += self.med.xml
        for a in self.arma:
            xml += a.xml
        xml += self.comb.xml
        xml += self.nRECOPI.xml
        xml += u'</prod>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cProd.xml = arquivo
            self.cEAN.xml = arquivo
            self.xProd.xml = arquivo
            self.NCM.xml = arquivo
            self.NVE.xml = arquivo
            self.CEST.xml = arquivo
            self.indEscala.xml = arquivo
            self.CNPJFab.xml = arquivo
            self.cBenef.xml = arquivo
            self.EXTIPI.xml = arquivo
            self.genero.xml = arquivo
            self.CFOP.xml = arquivo
            self.uCom.xml = arquivo
            self.qCom.xml = arquivo
            self.vUnCom.xml = arquivo
            self.vProd.xml = arquivo
            self.cEANTrib.xml = arquivo
            self.uTrib.xml = arquivo
            self.qTrib.xml = arquivo
            self.vUnTrib.xml = arquivo
            self.vFrete.xml = arquivo
            self.vSeg.xml = arquivo
            self.vDesc.xml = arquivo
            self.vOutro.xml = arquivo
            self.indTot.xml = arquivo
            self.DI = self.le_grupo(u'//det/prod/DI', DI)
            self.detExport = self.le_grupo(u'//det/prod/detExport', DetExport)
            self.xPed.xml = arquivo
            self.nItemPed.xml = arquivo
            self.nFCI.xml = arquivo
            self.rastro = self.le_grupo(u'//det/prod/rastro', Rastro)
            self.veicProd.xml = arquivo
            self.med.xml = arquivo
            self.arma = self.le_grupo(u'//det/prod/arma', Arma)
            self.comb.xml = arquivo
            self.nRECOPI.xml = arquivo


class Det(nfe_310.Det):
    def __init__(self):
        super(Det, self).__init__()
        self.prod = Prod()
        self.imposto = Imposto()
        self.impostoDevol = ImpostoDevolvido()

    # Se Simples Nacional nao tem CST, entao deve ser CSOSN
    def cst_formatado(self):
        formatado = unicode(self.imposto.ICMS.orig.valor).zfill(1)
        formatado += unicode(self.imposto.ICMS.CST.valor or self.imposto.ICMS.CSOSN.valor).zfill(2)

        return formatado

    @property
    def xml(self):
        xml = self.nItem.xml
        xml += self.prod.xml
        xml += self.imposto.xml
        xml += self.impostoDevol.xml
        xml += self.infAdProd.xml
        xml += u'</det>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nItem.xml = arquivo
            self.prod.xml = arquivo
            self.imposto.xml = arquivo
            self.impostoDevol.xml = arquivo
            self.infAdProd.xml = arquivo


class ICMSTot(nfe_310.ICMSTot):
    def __init__(self):
        super(ICMSTot, self).__init__()
        self.vFCP = TagDecimal(
            nome=u'vFCP', codigo=u'W04b', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//NFe/infNFe/total/ICMSTot', valor=u'0.00')
        self.vFCPST = TagDecimal(
            nome=u'vFCPST', codigo=u'W06a', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//NFe/infNFe/total/ICMSTot', valor=u'0.00')
        self.vFCPSTRet = TagDecimal(
            nome=u'vFCPSTRet', codigo=u'W06b', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//NFe/infNFe/total/ICMSTot', valor=u'0.00')
        self.vIPIDevol = TagDecimal(
            nome=u'vIPIDevol', codigo=u'W12a', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//NFe/infNFe/total/ICMSTot', valor=u'0.00')
        self.qBCMonoRet = TagDecimal(
            nome=u'qBCMonoRet', codigo=u'W06d.1', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//NFe/infNFe/total/ICMSTot', valor=u'0.00')
        self.vICMSMonoRet = TagDecimal(
            nome=u'vICMSMonoRet', codigo=u'W06e', tamanho=[1, 15, 1], decimais=[0, 2, 2],
            raiz=u'//NFe/infNFe/total/ICMSTot', valor=u'0.00')

    @property
    def xml(self):
        xml = u'<ICMSTot>'
        xml += self.vBC.xml
        xml += self.vICMS.xml
        xml += self.vICMSDeson.xml
        if self.vICMSUFDest.valor:
            xml += self.vFCPUFDest.xml
            xml += self.vICMSUFDest.xml
            xml += self.vICMSUFRemet.xml
        xml += self.vFCP.xml
        xml += self.vBCST.xml
        xml += self.vST.xml
        xml += self.vFCPST.xml
        xml += self.vFCPSTRet.xml
        if self.vICMSMonoRet.valor:
            xml += self.qBCMonoRet.xml
            xml += self.vICMSMonoRet.xml
        xml += self.vProd.xml
        xml += self.vFrete.xml
        xml += self.vSeg.xml
        xml += self.vDesc.xml
        xml += self.vII.xml
        xml += self.vIPI.xml
        xml += self.vIPIDevol.xml
        xml += self.vPIS.xml
        xml += self.vCOFINS.xml
        xml += self.vOutro.xml
        xml += self.vNF.xml
        xml += self.vTotTrib.xml
        xml += u'</ICMSTot>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.vBC.xml = arquivo
            self.vICMS.xml = arquivo
            self.vICMSDeson.xml = arquivo
            self.vFCPUFDest.xml = arquivo
            self.vICMSUFDest.xml = arquivo
            self.vICMSUFRemet.xml = arquivo
            self.vFCP.xml = arquivo
            self.vBCST.xml = arquivo
            self.vST.xml = arquivo
            self.vFCPST.xml = arquivo
            self.vFCPSTRet.xml = arquivo
            self.vProd.xml = arquivo
            self.vFrete.xml = arquivo
            self.vSeg.xml = arquivo
            self.vDesc.xml = arquivo
            self.vII.xml = arquivo
            self.vIPI.xml = arquivo
            self.vIPIDevol.xml = arquivo
            self.qBCMonoRet.xml = arquivo
            self.vICMSMonoRet.xml = arquivo
            self.vPIS.xml = arquivo
            self.vCOFINS.xml = arquivo
            self.vOutro.xml = arquivo
            self.vNF.xml = arquivo
            self.vTotTrib.xml = arquivo


class Total(nfe_310.Total):
    def __init__(self):
        super(Total, self).__init__()
        self.ICMSTot = ICMSTot()


class RefNF(nfe_310.RefNF):
    def __init__(self):
        super(RefNF, self).__init__()


class NFRef(nfe_310.NFRef):
    def __init__(self):
        super(NFRef, self).__init__()
        self.refNF = RefNF()


class Ide(nfe_310.Ide):
    def __init__(self):
        super(Ide, self).__init__()
        del self.indPag
        self.indIntermed = TagCaracter(
            nome=u'indIntermed', codigo=u'B25c', tamanho=[1, 1], raiz=u'//NFe/infNFe/ide',
            valor='0')

    @property
    def xml(self):
        xml = u'<ide>'
        xml += self.cUF.xml
        xml += self.cNF.xml
        xml += self.natOp.xml
        xml += self.mod.xml
        xml += self.serie.xml
        xml += self.nNF.xml
        xml += self.dhEmi.xml
        xml += self.dhSaiEnt.xml
        xml += self.tpNF.xml
        xml += self.idDest.xml
        xml += self.cMunFG.xml
        xml += self.tpImp.xml
        xml += self.tpEmis.xml
        xml += self.cDV.xml
        xml += self.tpAmb.xml
        xml += self.finNFe.xml
        xml += self.indFinal.xml
        xml += self.indPres.xml
        if self.indPres.valor in '12349':  # NT 2020.006
            if not self.indIntermed.valor:
                self.indIntermed.valor = '0'

            xml += self.indIntermed.xml

        xml += self.procEmi.xml
        xml += self.verProc.xml
        xml += self.dhCont.xml
        xml += self.xJust.xml
        for nr in self.NFref:
            xml += nr.xml
        xml += u'</ide>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.cUF.xml = arquivo
            self.cNF.xml = arquivo
            self.natOp.xml = arquivo
            self.mod.xml = arquivo
            self.serie.xml = arquivo
            self.nNF.xml = arquivo
            self.dhEmi.xml = arquivo
            self.dhSaiEnt.xml = arquivo
            self.tpNF.xml = arquivo
            self.idDest.xml = arquivo
            self.cMunFG.xml = arquivo
            self.tpImp.xml = arquivo
            self.tpEmis.xml = arquivo
            self.cDV.xml = arquivo
            self.tpAmb.xml = arquivo
            self.finNFe.xml = arquivo
            self.indFinal.xml = arquivo
            self.indPres.xml = arquivo
            self.indIntermed.xml = arquivo
            self.procEmi.xml = arquivo
            self.verProc.xml = arquivo
            self.dhCont.xml = arquivo
            self.xJust.xml = arquivo
            self.NFref = self.le_grupo(u'//NFe/infNFe/ide/NFref', NFRef)


class Card(nfe_310.Card):
    def __init__(self):
        super(Card, self).__init__()
        self.tpIntegra = TagInteiro(nome=u'tpIntegra', codigo=u'YA04a', tamanho=[1, 1, 1],
                                    raiz=u'//card')

    @property
    def xml(self):
        xml = u'<card>'
        xml += self.tpIntegra.xml
        if self.CNPJ.valor:
            xml += self.CNPJ.xml
        if self.tBand.valor:
            xml += self.tBand.xml
        if self.cAut.valor:
            xml += self.cAut.xml
        xml += u'</card>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpIntegra.xml = arquivo
            self.CNPJ.xml = arquivo
            self.tBand.xml = arquivo
            self.cAut.xml = arquivo


class DetPag(XMLNFe):
    def __init__(self):
        super(DetPag, self).__init__()
        self.indPag = TagCaracter(nome=u'indPag', codigo=u'YA01b', tamanho=[1, 1], raiz=u'//detPag',
                                  obrigatorio=False)
        self.tPag = TagCaracter(nome=u'tPag', codigo=u'YA02', tamanho=[2, 2],  raiz=u'//detPag')
        self.xPag = TagCaracter(nome=u'xPag', codigo=u'YA02a', tamanho=[2, 60],  raiz=u'//detPag', obrigatorio=False)
        self.vPag = TagDecimal(nome=u'vPag', codigo=u'YA03', tamanho=[1, 15, 1], decimais=[0, 2, 2],
                               raiz=u'//detPag')
        self.card = Card()

    @property
    def xml(self):
        xml = u'<detPag>'
        xml += self.indPag.xml
        xml += self.tPag.xml
        if self.tPag.valor in ('99'):
            xml += self.xPag.xml
        xml += self.vPag.xml
        if self.tPag.valor in ('03, 04'):
            xml += self.card.xml

        xml += u'</detPag>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.indPag.xml = arquivo
            self.tPag.xml = arquivo
            self.vPag.xml = arquivo
            self.xPag.xml = arquivo
            self.card.xml = arquivo


class Pag(nfe_310.Pag):
    def __init__(self):
        super(Pag, self).__init__()
        del self.tPag
        del self.vPag
        del self.card
        self.detPag = []
        self.vTroco = TagDecimal(nome=u'vTroco', codigo=u'YA09', tamanho=[1, 13, 1],
                                 decimais=[0, 13, 2], raiz=u'//pag', valor=u'0.00')

    @property
    def xml(self):
        xml = u'<pag>'
        for dp in self.detPag:
            xml += dp.xml
        if self.vTroco.valor:
            xml += self.vTroco.xml
        xml += u'</pag>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.detPag = self.le_grupo(u'//NFe/infNFe/pag/detPag', DetPag)
            self.vTroco.xml = arquivo


class AutXML(nfe_310.autXML):
    def __init__(self):
        super(AutXML, self).__init__()


class Emit(nfe_310.Emit):
    def __init__(self):
        super(Emit, self).__init__()

    @property
    def xml(self):
        xml = u'<emit>'
        xml += self.CNPJ.xml
        xml += self.CPF.xml
        xml += self.xNome.xml
        xml += self.xFant.xml
        xml += self.enderEmit.xml
        xml += self.IE.xml
        xml += self.IEST.xml
        if self.IM.valor:
            xml += self.IM.xml
            xml += self.CNAE.xml
        xml += self.CRT.xml
        xml += u'</emit>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.CPF.xml = arquivo
            self.xNome.xml = arquivo
            self.xFant.xml = arquivo
            self.enderEmit.xml = arquivo
            self.IE.xml = arquivo
            self.IEST.xml = arquivo
            self.IM.xml = arquivo
            self.CNAE.xml = arquivo
            self.CRT.xml = arquivo


class Dup(nfe_310.Dup):
    def __init__(self):
        super(Dup, self).__init__()
        self.vDup = TagDecimal(nome=u'vDup', codigo=u'Y10', tamanho=[1, 15, 1], decimais=[0, 2, 2],
                               raiz=u'//dup')

    @property
    def xml(self):
        if not (self.nDup.valor or self.dVenc.valor or self.vDup.valor):
            return u''
        xml = u'<dup>'
        xml += self.nDup.xml
        xml += self.dVenc.xml
        xml += self.vDup.xml
        xml += u'</dup>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nDup.xml = arquivo
            self.dVenc.xml = arquivo
            self.vDup.xml = arquivo


class Fat(XMLNFe):
    def __init__(self):
        super(Fat, self).__init__()
        self.nFat = TagCaracter(nome=u'nFat', codigo=u'Y03', tamanho=[1, 60],
                                raiz=u'//NFe/infNFe/cobr/fat', obrigatorio=False)
        self.vOrig = TagDecimal(nome=u'vOrig', codigo=u'Y04', tamanho=[1, 15, 1],
                                decimais=[0, 2, 2], raiz=u'//NFe/infNFe/cobr/fat',
                                obrigatorio=False)
        self.vDesc = TagDecimal(nome=u'vDesc', codigo=u'Y05', tamanho=[1, 15, 1],
                                decimais=[0, 2, 2], raiz=u'//NFe/infNFe/cobr/fat',
                                obrigatorio=False)
        self.vLiq = TagDecimal(nome=u'vLiq', codigo=u'Y06', tamanho=[1, 15, 1], decimais=[0, 2, 2],
                               raiz=u'//NFe/infNFe/cobr/fat', obrigatorio=False)

    @property
    def xml(self):
        if not self.nFat.valor:
            return u''
        xml = u'<fat>'
        xml += self.nFat.xml
        xml += self.vOrig.xml
        xml += self.vDesc.xml
        xml += self.vLiq.xml
        xml += u'</fat>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nFat.xml = arquivo
            self.vOrig.xml = arquivo
            self.vDesc.xml = arquivo
            self.vLiq.xml = arquivo


class Cobr(XMLNFe):
    def __init__(self):
        super(Cobr, self).__init__()
        self.fat = Fat()
        self.dup = []

    @property
    def xml(self):
        if not any((self.fat.nFat.valor, self.dup)):
            return u''
        xml = u'<cobr>'
        xml += self.fat.xml
        for d in self.dup:
            xml += d.xml
        xml += u'</cobr>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.fat.xml = arquivo
            self.dup = self.le_grupo(u'//NFe/infNFe/cobr/dup', Dup)


class Dest(nfe_310.Dest):
    def __init__(self):
        super(Dest, self).__init__()

    @property
    def xml(self):
        if not self.enderDest.xLgr.valor:
            return u''
        xml = u'<dest>'
        xml += self.CPF.xml if self.CPF.valor else self.CNPJ.xml
        xml += self.idEstrangeiro.xml
        xml += self.xNome.xml
        xml += self.enderDest.xml
        xml += self.indIEDest.xml
        xml += self.IE.xml
        xml += self.ISUF.xml
        xml += self.email.xml
        xml += u'</dest>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        self.CNPJ.xml = arquivo
        self.CPF.xml = arquivo
        self.idEstrangeiro.xml = arquivo
        self.xNome.xml = arquivo
        self.enderDest.xml = arquivo
        self.indIEDest.xml = arquivo
        self.IE.xml = arquivo
        self.ISUF.xml = arquivo
        self.email.xml = arquivo


class InfRespTec(XMLNFe):
    """Informacoes do responsavel tecnico"""
    def __init__(self):
        super(InfRespTec, self).__init__()
        self.CNPJ = TagCaracter(
            nome=u'CNPJ', codigo=u'ZD02', tamanho=[14, 14], raiz=u'//NFe/infNFe',
            valor=u'01526558000178')
        self.xContato = TagCaracter(
            nome=u'xContato', codigo=u'ZD04', tamanho=[2, 60], raiz=u'//NFe/infNFe',
            valor=u'Luiz Fernando Maciel Franca')
        self.email = TagCaracter(
            nome=u'email', codigo=u'ZD05', tamanho=[6, 60], raiz=u'//NFe/infNFe',
            valor=u'luiz@siginformatica.com.br')
        self.fone = TagCaracter(
            nome=u'fone', codigo=u'ZD06', tamanho=[6, 14], raiz=u'//NFe/infNFe',
            valor=u'3121061700')
        self.idCSRT = TagCaracter(
            nome=u'idCSRT', codigo=u'ZD08', tamanho=[2, 2], raiz=u'//NFe/infNFe',
            valor=u'01')
        self.hashCSRT = TagCaracter(
            nome=u'hashCSRT', codigo=u'ZD09', tamanho=[28, 28], raiz=u'//NFe/infNFe')

    @property
    def xml(self):
        if not self.hashCSRT.valor:
            return u''
        xml = u'<infRespTec>'
        xml += self.CNPJ.xml
        xml += self.xContato.xml
        xml += self.email.xml
        xml += self.fone.xml
        xml += self.idCSRT.xml
        xml += self.hashCSRT.xml
        xml += u'</infRespTec>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.xContato.xml = arquivo
            self.email.xml = arquivo
            self.fone.xml = arquivo
            self.idCSRT.xml = arquivo
            self.hashCSRT.xml = arquivo


class InfIntermed(XMLNFe):
    def __init__(self):
        super(InfIntermed, self).__init__()
        self.CNPJ = TagCaracter(
            nome=u'CNPJ', codigo=u'YB02', tamanho=[14, 14], raiz=u'//NFe/infNFe/infIntermed')
        self.idCadIntTran = TagCaracter(
            nome=u'idCadIntTran', codigo=u'YB03', tamanho=[0, 60], raiz=u'//NFe/infNFe/infIntermed')

    @property
    def xml(self):
        xml = XMLNFe.get_xml(self)
        if self.CNPJ.valor or self.idCadIntTran.valor:
            xml += u'<infIntermed>'
            xml += self.CNPJ.xml
            xml += self.idCadIntTran.xml
            xml += u'</infIntermed>'
        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml = arquivo
            self.idCadIntTran.xml = arquivo


class InfNFe(nfe_310.InfNFe):
    def __init__(self):
        super(InfNFe, self).__init__()
        self.versao = TagDecimal(
            nome=u'infNFe', codigo=u'A01', propriedade=u'versao', raiz=u'//NFe',
            namespace=NAMESPACE_NFE, valor=u'4.00')
        self.ide = Ide()
        self.emit = Emit()
        self.dest = Dest()
        self.det = []
        self.total = Total()
        self.cobr = Cobr()
        self.pag = Pag()
        self.infIntermed = InfIntermed()
        self.infRespTec = InfRespTec()

    @property
    def xml(self):
        xml = u'<infNFe versao="%s" Id="%s">' % (self.versao.valor, self.Id.valor)
        xml += self.ide.xml
        xml += self.emit.xml
        xml += self.avulsa.xml
        xml += self.dest.xml
        xml += self.retirada.xml
        xml += self.entrega.xml
        for aut in self.autXML:
            xml += aut.xml
        for d in self.det:
            xml += d.xml
        xml += self.total.xml
        xml += self.transp.xml
        xml += self.cobr.xml
        xml += self.pag.xml
        xml += self.infIntermed.xml
        xml += self.infAdic.xml
        xml += self.exporta.xml
        xml += self.compra.xml
        xml += self.cana.xml
        xml += self.infRespTec.xml
        xml += u'</infNFe>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.Id.xml = arquivo
            self.ide.xml = arquivo
            self.emit.xml = arquivo
            self.avulsa.xml = arquivo
            self.dest.xml = arquivo
            self.retirada.xml = arquivo
            self.entrega.xml = arquivo
            self.autXML = self.le_grupo(u'//NFe/infNFe/autXML', AutXML)
            self.det = self.le_grupo(u'//NFe/infNFe/det', Det)
            self.total.xml = arquivo
            self.transp.xml = arquivo
            self.cobr.xml = arquivo
            self.pag.xml = arquivo
            self.infIntermed.xml = arquivo
            self.infAdic.xml = arquivo
            self.exporta.xml = arquivo
            self.compra.xml = arquivo
            self.cana.xml = arquivo
            self.infRespTec.xml = arquivo


class NFe(nfe_310.NFe):
    def __init__(self):
        super(NFe, self).__init__()
        self.infNFe = InfNFe()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/', ESQUEMA_ATUAL + u'/')
        self.arquivo_esquema = u'nfe_v4.00.xsd'

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
