# -*- coding: utf-8 -*-

import httplib
import socket
import ssl


class HTTPSSignedConnection(httplib.HTTPSConnection):
    """ Class to make a HTTPS connection, signed by a valid/trusted CA root certificate, 
        with support for full client-based SSL/TLS Authentication"""

    def __init__(self, host, port=None, key_file=None, cert_file=None, ca_file=None, timeout=10):
        httplib.HTTPSConnection.__init__(self, host, key_file=key_file, cert_file=cert_file, timeout=timeout)
        self.ca_file = ca_file

    def connect(self):
        """ Connect to a host on a given (SSL) port.
            If ca_file is pointing somewhere, use it to check Server Certificate.

            Redefined/copied and extended from httplib.py:1105 (Python 2.6.x).
            This is needed to pass cert_reqs=ssl.CERT_REQUIRED as parameter to ssl.wrap_socket(),
            which forces SSL to check server certificate against our client certificate.
        """
        if hasattr(self, 'source_address'):
            sock = socket.create_connection((self.host, self.port), self.timeout,
                                            self.source_address)
        else:
            sock = socket.create_connection((self.host, self.port), self.timeout)
        if self._tunnel_host:
            self.sock = sock
            self._tunnel()

        if hasattr(self, 'ca_file'):
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file, ca_certs=self.ca_file,
                                           cert_reqs=ssl.CERT_REQUIRED, ssl_version=ssl.PROTOCOL_TLSv1_2,
                                           do_handshake_on_connect=False)
        else:
            # Fallback to unsigned HTTPSConnection
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file, ca_certs=None,
                                           cert_reqs=ssl.CERT_NONE, ssl_version=ssl.PROTOCOL_TLSv1_2,
                                           do_handshake_on_connect=False)

        pass

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
