# -*- encoding: utf-8 -*-
# #############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
# www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################


from nfe.pysped.xml_sped import *

import os

DIRNAME = os.path.dirname(__file__)


class CodServico(XMLNFe):
    def __init__(self):
        super(CodServico, self).__init__()
        self.codservico = TagCaracter(
            nome=u'codservico', codigo=u'', tamanho=[1, 15], raiz=u'/codservico')

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '\t\t' + self.codservico.xml + '\n'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.codservico.xml = arquivo

    xml = property(get_xml, set_xml)


class Importacao(XMLNFe):
    def __init__(self):
        super(Importacao, self).__init__()
        self.rps_numero = TagCaracter(nome=u'rps_numero', codigo=u'',  tamanho=[0, 20], raiz=u'//importacao')#, obrigatorio=False)
        self.rps_data = TagData(nome=u'rps_data', codigo=u'', raiz=u'//importacao/nota')#, obrigatorio=False)
        self.tomador_nome = TagCaracter(nome=u'tomador_nome', codigo=u'', tamanho=[0, 100], raiz=u'//importacao/nota')
        self.tomador_cnpjcpf = TagCaracter(nome=u'tomador_cnpjcpf', codigo=u'', tamanho=[13, 18], raiz=u'//importacao/nota')
        self.tomador_inscrmunicipal = TagCaracter(nome=u'tomador_inscrmunicipal', codigo=u'', tamanho=[0, 20], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.tomador_endereco = TagCaracter(nome=u'tomador_endereco', codigo=u'', tamanho=[0, 40], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.tomador_cep = TagCaracter(nome=u'tomador_cep', codigo=u'', tamanho=[1, 12], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.tomador_municipio = TagCaracter(nome=u'tomador_municipio', codigo=u'', tamanho=[1, 100], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.tomador_uf = TagCaracter(nome=u'tomador_uf', codigo=u'', tamanho=[0, 3], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.tomador_email = TagCaracter(nome=u'tomador_email', codigo=u'', tamanho=[1, 100], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.discriminacao = TagCaracter(nome=u'discriminacao', codigo=u'', tamanho=[1, 400], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.valortotal = TagDecimal(nome=u'valortotal', codigo=u'', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.valordeducoes = TagDecimal(nome=u'valordeducoes', codigo=u'', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.basecalculo = TagDecimal(nome=u'basecalculo', codigo=u'', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.valoriss = TagDecimal(nome=u'valoriss', codigo=u'', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.issretido = TagDecimal(nome=u'issretido', codigo=u'', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.credito = TagDecimal(nome=u'credito', codigo=u'', tamanho=[1, 10, 1], decimais=[0, 2, 2], raiz=u'//importacao/nota')#, obrigatorio=False)
        self.estado = TagCaracter(nome=u'estado', codigo=u'', tamanho=[1, 20], raiz=u'//importacao/nota')
        self.codservico = []  # CodServico()

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<?xml version="1.0" encoding="iso-8859-1"?>\n'
        xml += u'<importacao>\n'
        xml += u'\t<nota>\n'
        xml += '\t\t' + self.rps_numero.xml + '\n'
        xml += '\t\t' + self.rps_data.xml + '\n'
        xml += '\t\t' + self.tomador_nome.xml + '\n'
        xml += '\t\t' + self.tomador_cnpjcpf.xml + '\n'
        xml += '\t\t' + self.tomador_inscrmunicipal.xml + '\n'
        xml += '\t\t' + self.tomador_endereco.xml + '\n'
        xml += '\t\t' + self.tomador_cep.xml + '\n'
        xml += '\t\t' + self.tomador_municipio.xml + '\n'
        xml += '\t\t' + self.tomador_uf.xml + '\n'
        xml += '\t\t' + self.tomador_email.xml + '\n'
        xml += '\t\t' + self.discriminacao.xml + '\n'
        xml += '\t\t' + self.valortotal.xml + '\n'
        xml += '\t\t' + self.valordeducoes.xml + '\n'
        xml += '\t\t' + self.basecalculo.xml + '\n'
        xml += '\t\t' + self.valoriss.xml + '\n'
        xml += '\t\t' + self.issretido.xml + '\n'
        xml += '\t\t' + self.credito.xml + '\n'
        xml += '\t\t' + self.estado.xml + '\n'

        for d in self.codservico:
            xml += d.xml

        xml += '\t' + u'</nota>' + '\n'
        xml += u'</importacao>'

        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.rps_numero.xml = arquivo
            self.rps_data.xml = arquivo
            self.tomador_nome.xml = arquivo
            self.tomador_cnpjcpf.xml = arquivo
            self.tomador_inscrmunicipal.xml = arquivo
            self.tomador_endereco.xml = arquivo
            self.tomador_cep.xml = arquivo
            self.tomador_municipio.xml = arquivo
            self.tomador_uf.xml = arquivo
            self.tomador_email.xml = arquivo
            self.discriminacao.xml = arquivo
            self.valortotal.xml = arquivo
            self.valordeducoes.xml = arquivo
            self.basecalculo.xml = arquivo
            self.valoriss.xml = arquivo
            self.issretido.xml = arquivo
            self.credito.xml = arquivo
            self.estado.xml = arquivo
            self.ret = self.le_grupo('//importacao/nota/codservico', CodServico)

    xml = property(get_xml, set_xml)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: