# -*- coding: utf-8 -*-

from nfe.pysped.xml_sped.base_nfse import *


DIRNAME = os.path.dirname(__file__)

# -------------------- Tipos Complexos -------------------

class Cabecalho(XMLNFSe):
    def __init__(self):
        super(Cabecalho, self).__init__()
        self.nome_tag = 'cabecalho'
        self.Versao = TagInteiro(nome=u'Versao', codigo=u'Versao', tamanho=[1,4,1],
                                 namespace=NAMESPACE_NFSE)
        self.versaoDados = TagInteiro(nome=u'versaoDados', codigo=u'versaoDados', tamanho=[1,4,1])

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<?xml version="1.0"?>')
        xml.append(u'<%s Versao="%s">' % (self.nome_tag, self.Versao.xml))
        xml.append(self.versaoDados.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Versao.xml = arquivo
            self.versaoDados.xml = arquivo

    xml = property(get_xml, set_xml)

class Signature(XMLNFSe):
    def __init__(self):
        super(Signature, self).__init__()
        self.URI = u''
        self.Id = u''
        self.DigestValue = u''
        self.SignatureValue = u''
        self.X509Certificate = u''
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/')
        self.arquivo_esquema = u'xmldsig-core-schema_v1.01.xsd'

    @property
    def xml(self):
        if not len(self.URI):
            self.URI = u'#'
        if self.URI[0] != u'#':
            self.URI = u'#' + self.URI
        xml = u'<Signature xmlns="http://www.w3.org/2000/09/xmldsig#" Id="%s">' % self.Id
        xml += u'<SignedInfo>'
        xml += u'<CanonicalizationMethod Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>'
        xml += u'<SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#rsa-sha1"/>'
        xml += u'<Reference URI="' + self.URI + u'">'
        xml += u'<Transforms>'
        xml += u'<Transform Algorithm="http://www.w3.org/2000/09/xmldsig#enveloped-signature"/>'
        xml += u'<Transform Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>'
        xml += u'</Transforms>'
        xml += u'<DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>'
        xml += u'<DigestValue>' + self.DigestValue + u'</DigestValue>'
        xml += u'</Reference>'
        xml += u'</SignedInfo>'
        xml += u'<SignatureValue>' + self.SignatureValue + u'</SignatureValue>'
        xml += u'<KeyInfo>'
        xml += u'<X509Data>'
        xml += u'<X509Certificate>' + self.X509Certificate + u'</X509Certificate>'
        xml += u'</X509Data>'
        xml += u'</KeyInfo>'
        xml += u'</Signature>'

        return xml

    @xml.setter
    def xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id = self._le_tag(u'//sig:Signature', u'Id') or u''
            self.URI = self._le_tag(u'//sig:Signature/sig:SignedInfo/sig:Reference', u'URI') or u''
            self.DigestValue = self._le_tag(
                u'//sig:Signature/sig:SignedInfo/sig:Reference/sig:DigestValue') or u''
            self.SignatureValue = self._le_tag(u'//sig:Signature/sig:SignatureValue') or u''
            self.X509Certificate = self._le_tag(
                u'//sig:Signature/sig:KeyInfo/sig:X509Data/sig:X509Certificate') or u''

class tcCpfCnpj(XMLNFSe):
    """
    Número de CPF ou CNPJ
    """
    def __init__(self, raiz=''):
        super(tcCpfCnpj, self).__init__()
        self.nome_tag = 'CpfCnpj'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Cpf = tsCpf(nome=u'Cpf', codigo=u'Cpf', raiz=str_raiz, obrigatorio=False)
        self.Cnpj = tsCnpj(nome=u'Cnpj', codigo=u'Cnpj', raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        if not self.Cpf.valor and not self.Cnpj.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        if self.Cpf.valor:
            xml.append(self.Cpf.xml)
        else:
            xml.append(self.Cnpj.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Cpf.xml = arquivo
            self.Cnpj.xml = arquivo

    xml = property(get_xml, set_xml)

class tcEndereco(XMLNFSe):
    """
    Representação completa do endereço
    """
    def __init__(self, raiz=''):
        super(tcEndereco, self).__init__()
        self.nome_tag = 'Endereco'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Endereco = tsEndereco(nome=u'Endereco', codigo=u'Endereco', raiz=str_raiz,
                                   obrigatorio=False)
        self.Numero = tsNumeroEndereco(nome=u'Numero', codigo=u'Numero', raiz=str_raiz,
                                       obrigatorio=False)
        self.Complemento = tsComplementoEndereco(nome=u'Complemento', codigo=u'Complemento',
                                                 raiz=str_raiz, obrigatorio=False)
        self.Bairro = tsBairro(nome=u'Bairro', codigo=u'Bairro', raiz=str_raiz, obrigatorio=False)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome=u'CodigoMunicipio',
                                                     codigo=u'CodigoMunicipio', raiz=str_raiz,
                                                     obrigatorio=False)
        self.Uf = tsUf(nome=u'Uf', codigo=u'Uf', raiz=str_raiz, obrigatorio=False)
        self.CodigoPais = tsCodigoPaisBacen(nome=u'CodigoPais', codigo=u'CodigoPais', raiz=str_raiz,
                                            obrigatorio=False)
        self.Cep = tsCep(nome=u'Cpf', codigo=u'Cpf', raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Endereco.xml)
        xml.append(self.Numero.xml)
        xml.append(self.Complemento.xml)
        xml.append(self.Bairro.xml)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(self.Uf.xml)
        xml.append(self.CodigoPais.xml)
        xml.append(self.Cep.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Endereco.xml = arquivo
            self.Numero.xml = arquivo
            self.Complemento.xml = arquivo
            self.Bairro.xml = arquivo
            self.CodigoMunicipio.xml = arquivo
            self.Uf.xml = arquivo
            self.CodigoPais.xml = arquivo
            self.Cep.xml = arquivo

    xml = property(get_xml, set_xml)

class tcContato(XMLNFSe):
    """
    Representa forma de contato com a pessoa (física/jurídica)
    """
    def __init__(self, raiz=''):
        super(tcContato, self).__init__()
        self.nome_tag = 'Contato'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Telefone = tsTelefone(nome=u'Telefone', codigo=u'Telefone', raiz=str_raiz,
                                   obrigatorio=False)
        self.Email = tsEmail(nome=u'Email', codigo=u'Email', raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        if not self.Telefone or not self.Email:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Telefone.xml)
        xml.append(self.Email.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Telefone.xml = arquivo
            self.Email.xml = arquivo

    xml = property(get_xml, set_xml)

class tcIdentificacaoOrgaoGerador(XMLNFSe):
    """
    Representa dados para identificação de órgão gerador
    """
    def __init__(self, raiz=''):
        super(tcIdentificacaoOrgaoGerador, self).__init__()
        self.nome_tag = 'OrgaoGerador'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome=u'CodigoMunicipio',
                                                     codigo=u'CodigoMunicipio', raiz=str_raiz)
        self.Uf = tsUf(nome=u'Uf', codigo=u'Uf', raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(self.Uf.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CodigoMunicipio.xml = arquivo
            self.Uf.xml = arquivo

    xml = property(get_xml, set_xml)

class tcIdentificacaoRps(XMLNFSe):
    """
    Dados de identificação do RPS
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoRps, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Numero = tsNumeroRps(nome=u'Numero', codigo=u'Numero', raiz=str_raiz)
        self.Serie = tsSerieRps(nome=u'Serie', codigo=u'Serie', raiz=str_raiz)
        self.Tipo = tsTipoRps(nome=u'Tipo', codigo=u'Tipo', raiz=str_raiz)

    def get_xml(self):
        if not self.Numero.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Numero.xml)
        xml.append(self.Serie.xml)
        xml.append(self.Tipo.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Numero.xml = arquivo
            self.Serie.xml = arquivo
            self.Tipo.xml = arquivo

    xml = property(get_xml, set_xml)

class tcIdentificacaoPrestador(XMLNFSe):
    """
    Representa dados para identificação do prestador de serviço
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoPrestador, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome=u'InscricaoMunicipal',
                                                       codigo=u'InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo

    xml = property(get_xml, set_xml)

class tcIdentificacaoTomador(XMLNFSe):
    """
    Representa dados para identificação do tomador de serviço
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoTomador, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome=u'InscricaoMunicipal',
                                                       codigo=u'InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        if not self.CpfCnpj.xml:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo

    xml = property(get_xml, set_xml)

class tcIdentificacaoConsulente(XMLNFSe):
    """
    Representa dados para identificação do prestador de serviço
    """
    def __init__(self, raiz=''):
        super(tcIdentificacaoConsulente, self).__init__()
        self.nome_tag = 'Consulente'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome=u'InscricaoMunicipal',
                                                       codigo=u'InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo

    xml = property(get_xml, set_xml)

class tcIdentificacaoIntermediario(XMLNFSe):
    """
    Representa dados para identificação do tomador de serviço
    """
    def __init__(self, nome, raiz=''):
        super(tcIdentificacaoIntermediario, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome=u'InscricaoMunicipal',
                                                       codigo=u'InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        if not self.CpfCnpj.xml:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo

    xml = property(get_xml, set_xml)

class tcDadosTomador(XMLNFSe):
    """
    Representa dados do tomador de serviço
    """
    def __init__(self, raiz=''):
        super(tcDadosTomador, self).__init__()
        self.nome_tag = 'Tomador'  # Todo: Na documentacao a tag eh 'TomadorServico' diferente do xsd
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoTomador = tcIdentificacaoTomador(nome=u'IdentificacaoTomador',
                                                           raiz=str_raiz)
        self.RazaoSocial = tsRazaoSocial(nome=u'RazaoSocial', codigo=u'RazaoSocial',
                                         raiz=str_raiz, obrigatorio=False)
        self.Endereco = tcEndereco(raiz=str_raiz)
        self.Contato = tcContato(raiz=str_raiz)

    def get_xml(self):
        if not self.IdentificacaoTomador.xml:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.IdentificacaoTomador.xml)
        xml.append(self.RazaoSocial.xml)
        xml.append(self.Endereco.xml)
        xml.append(self.Contato.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoTomador.xml = arquivo
            self.RazaoSocial.xml = arquivo
            self.Endereco.xml = arquivo
            self.Contato.xml = arquivo

    xml = property(get_xml, set_xml)

class tcDadosIntermediario(XMLNFSe):
    """
    Representa dados para identificação de intermediário do serviço
    """
    def __init__(self, raiz=''):
        super(tcDadosIntermediario, self).__init__()
        self.nome_tag = 'Intermediario'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoIntermediario = tcIdentificacaoIntermediario(
                                                  nome=u'IdentificacaoIntermediario', raiz=str_raiz)
        self.RazaoSocial = tsRazaoSocial(nome=u'RazaoSocial', codigo=u'RazaoSocial', raiz=str_raiz)

    def get_xml(self):
        if not self.IdentificacaoIntermediario.xml:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.IdentificacaoIntermediario.xml)
        xml.append(self.RazaoSocial.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoIntermediario.xml = arquivo
            self.RazaoSocial.xml = arquivo

    xml = property(get_xml, set_xml)

class tcValoresDeclaracaoServico(XMLNFSe):
    """
    Representa um conjunto de valores que compõe a declaração do serviço
    """
    def __init__(self, raiz=''):
        super(tcValoresDeclaracaoServico, self).__init__()
        self.nome_tag = 'Valores'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.ValorServicos = tsValor(nome=u'ValorServicos', codigo=u'ValorServicos', raiz=str_raiz)
        self.ValorDeducoes = tsValor(nome=u'ValorDeducoes', codigo=u'ValorDeducoes', raiz=str_raiz,
                                     obrigatorio=False)
        self.ValorPis = tsValor(nome=u'ValorPis', codigo=u'ValorPis', raiz=str_raiz,
                                obrigatorio=False)
        self.ValorCofins = tsValor(nome=u'ValorCofins', codigo=u'ValorCofins', raiz=str_raiz,
                                   obrigatorio=False)
        self.ValorInss = tsValor(nome=u'ValorInss', codigo=u'ValorInss', raiz=str_raiz,
                                         obrigatorio=False)
        self.ValorIr = tsValor(nome=u'ValorIr', codigo=u'ValorIr', raiz=str_raiz,
                                         obrigatorio=False)
        self.ValorCsll = tsValor(nome=u'ValorCsll', codigo=u'ValorCsll', raiz=str_raiz,
                                         obrigatorio=False)
        self.OutrasRetencoes = tsValor(nome=u'OutrasRetencoes', codigo=u'OutrasRetencoes',
                                       raiz=str_raiz, obrigatorio=False)
        self.ValorIss = tsValor(nome=u'ValorIss', codigo=u'ValorIss', raiz=str_raiz,
                                         obrigatorio=False)
        self.Aliquota = tsAliquota(nome=u'Aliquota', codigo=u'Aliquota', raiz=str_raiz,
                                         obrigatorio=False)
        self.DescontoIncondicionado = tsValor(nome=u'DescontoIncondicionado',
                                              codigo=u'DescontoIncondicionado', raiz=str_raiz,
                                              obrigatorio=False)
        self.DescontoCondicionado = tsValor(nome=u'DescontoCondicionado',
                                            codigo=u'DescontoCondicionado', raiz=str_raiz,
                                            obrigatorio=False)

    def get_xml(self):
        if not self.ValorServicos.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.ValorServicos.xml)
        xml.append(self.ValorDeducoes.xml)
        xml.append(self.ValorPis.xml)
        xml.append(self.ValorCofins.xml)
        xml.append(self.ValorInss.xml)
        xml.append(self.ValorIr.xml)
        xml.append(self.ValorCsll.xml)
        xml.append(self.OutrasRetencoes.xml)
        xml.append(self.ValorIss.xml)
        xml.append(self.Aliquota.xml)
        xml.append(self.DescontoIncondicionado.xml)
        xml.append(self.DescontoCondicionado.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.ValorServicos.xml = arquivo
            self.ValorDeducoes.xml = arquivo
            self.ValorPis.xml = arquivo
            self.ValorCofins.xml = arquivo
            self.ValorInss.xml = arquivo
            self.ValorIr.xml = arquivo
            self.ValorCsll.xml = arquivo
            self.OutrasRetencoes.xml = arquivo
            self.ValorIss.xml = arquivo
            self.Aliquota.xml = arquivo
            self.DescontoIncondicionado.xml = arquivo
            self.DescontoCondicionado.xml = arquivo

    xml = property(get_xml, set_xml)

class tcValoresNfse(XMLNFSe):
    """
    Representa um conjunto de valores que compõe o documento fiscal
    """
    def __init__(self, raiz=''):
        super(tcValoresNfse, self).__init__()
        self.nome_tag = 'ValoresNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.BaseCalculo = tsValor(nome=u'BaseCalculo', codigo=u'BaseCalculo', raiz=str_raiz,
                                   obrigatorio=False)
        self.Aliquota = tsAliquota(nome=u'Aliquota', codigo=u'Aliquota', raiz=str_raiz,
                                   obrigatorio=False)
        self.ValorIss = tsValor(nome=u'ValorIss', codigo=u'ValorIss', raiz=str_raiz,
                                obrigatorio=False)
        self.ValorLiquidoNfse = tsValor(nome=u'ValorLiquidoNfse', codigo=u'ValorLiquidoNfse',
                                        raiz=str_raiz) # valor = (ValorServicos - ValorPIS -
                                                       #          ValorCOFINS - ValorINSS -
                                                       #          ValorIR - ValorCSLL -
                                                       #          OutrasRetençoes -
                                                       #          ValorISSRetido -
                                                       #          DescontoIncondicionado -
                                                       #          DescontoCondicionado)

    def get_xml(self):
        if not self.ValorLiquidoNfse.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.BaseCalculo.xml)
        xml.append(self.Aliquota.xml)
        xml.append(self.ValorIss.xml)
        xml.append(self.ValorLiquidoNfse.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.BaseCalculo.xml = arquivo
            self.Aliquota.xml = arquivo
            self.ValorIss.xml = arquivo
            self.ValorLiquidoNfse.xml = arquivo

    xml = property(get_xml, set_xml)

class tcDadosServico(XMLNFSe):
    """
    Representa dados que compõe o serviço prestado
    """
    def __init__(self, raiz=''):
        super(tcDadosServico, self).__init__()
        self.nome_tag = 'Servico'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Valores = tcValoresDeclaracaoServico(raiz=str_raiz)
        self.IssRetido = tsSimNao(nome=u'IssRetido', codigo=u'IssRetido', raiz=str_raiz)
        self.ResponsavelRetencao = tsResponsavelRetencao(nome=u'ResponsavelRetencao',
                                                         codigo=u'ResponsavelRetencao',
                                                         raiz=str_raiz, obrigatorio=False)
        self.ItemListaServico = tsItemListaServico(nome=u'ItemListaServico',
                                                   codigo=u'ItemListaServico', raiz=str_raiz)
        self.CodigoCnae = tsCodigoCnae(nome=u'CodigoCnae', codigo=u'CodigoCnae', raiz=str_raiz,
                                       obrigatorio=False)
        self.CodigoTributacaoMunicipio = tsCodigoTributacao(nome=u'CodigoTributacaoMunicipio',
                              codigo=u'CodigoTributacaoMunicipio', raiz=str_raiz, obrigatorio=False)
        self.Discriminacao = tsDiscriminacao(nome=u'Discriminacao', codigo=u'Discriminacao',
                                             raiz=str_raiz)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome=u'CodigoMunicipio',
                                                     codigo=u'CodigoMunicipio', raiz=str_raiz)
        self.CodigoPais = tsCodigoPaisBacen(nome=u'CodigoPais', codigo=u'CodigoPais', raiz=str_raiz,
                                            obrigatorio=False)
        self.ExigibilidadeISS = tsExigibilidadeIss(nome=u'ExigibilidadeISS',
                                                   codigo=u'ExigibilidadeISS', raiz=str_raiz)
        self.MunicipioIncidencia = tsCodigoMunicipioIbge(nome=u'MunicipioIncidencia',
                                    codigo=u'MunicipioIncidencia', raiz=str_raiz, obrigatorio=False)
        self.NumeroProcesso = tsNumeroProcesso(nome=u'NumeroProcesso', codigo=u'NumeroProcesso',
                                               raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Valores.xml)
        xml.append(self.IssRetido.xml)
        xml.append(self.ResponsavelRetencao.xml)
        xml.append(self.ItemListaServico.xml)
        xml.append(self.CodigoCnae.xml)
        xml.append(self.CodigoTributacaoMunicipio.xml)
        xml.append(self.Discriminacao.xml)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(self.CodigoPais.xml)
        xml.append(self.ExigibilidadeISS.xml)
        xml.append(self.MunicipioIncidencia.xml)
        xml.append(self.NumeroProcesso.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Valores.xml = arquivo
            self.IssRetido.xml = arquivo
            self.ResponsavelRetencao.xml = arquivo
            self.ItemListaServico.xml = arquivo
            self.CodigoCnae.xml = arquivo
            self.CodigoTributacaoMunicipio.xml = arquivo
            self.Discriminacao.xml = arquivo
            self.CodigoMunicipio.xml = arquivo
            self.CodigoPais.xml = arquivo
            self.ExigibilidadeISS.xml = arquivo
            self.MunicipioIncidencia.xml = arquivo
            self.NumeroProcesso.xml = arquivo

    xml = property(get_xml, set_xml)

class tcDadosConstrucaoCivil(XMLNFSe):
    """
    Representa dados para identificação de construção civil
    """
    def __init__(self, raiz=''):
        super(tcDadosConstrucaoCivil, self).__init__()
        self.nome_tag = 'ConstrucaoCivil'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CodigoObra = tsCodigoObra(nome=u'CodigoObra', codigo=u'CodigoObra', raiz=str_raiz,
                                       obrigatorio=False)
        self.Art = tsArt(nome=u'Art', codigo=u'Art', raiz=str_raiz)

    def get_xml(self):
        if not self.Art.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CodigoObra.xml)
        xml.append(self.Art.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CodigoObra.xml = arquivo
            self.Art.xml = arquivo

    xml = property(get_xml, set_xml)

class tcDadosPrestador(XMLNFSe):
    """
    Representa dados do prestador do serviço
    """
    def __init__(self, raiz=''):
        super(tcDadosPrestador, self).__init__()
        self.nome_tag = 'PrestadorServico'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoPrestador = tcIdentificacaoPrestador(nome=u'IdentificacaoPrestador',
                                                               codigo=u'IdentificacaoPrestador',
                                                               raiz=str_raiz)
        self.RazaoSocial = tsRazaoSocial(nome=u'RazaoSocial', codigo=u'RazaoSocial', raiz=str_raiz)
        self.NomeFantasia = tsNomeFantasia(nome=u'NomeFantasia', codigo=u'NomeFantasia',
                                           raiz=str_raiz, obrigatorio=False)
        self.Endereco = tcEndereco(raiz=str_raiz)
        self.Contato = tcContato(raiz=str_raiz)

    def get_xml(self):
        if not self.IdentificacaoPrestador.valor and not self.RazaoSocial.valor and \
                self.Endereco.xml:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.IdentificacaoPrestador.xml)
        xml.append(self.RazaoSocial.xml)
        xml.append(self.NomeFantasia.xml)
        xml.append(self.Endereco.xml)
        xml.append(self.Contato.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoPrestador.xml = arquivo
            self.RazaoSocial.xml = arquivo
            self.NomeFantasia.xml = arquivo
            self.Endereco.xml = arquivo
            self.Contato.xml = arquivo

    xml = property(get_xml, set_xml)

class tcInfRps(XMLNFSe):
    """
    Representa dados informativos do Recibo Provisório de Serviço (RPS)
    """
    def __init__(self, raiz=''):
        super(tcInfRps, self).__init__()
        self.nome_tag = 'Rps'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'Rps', codigo=u'Id', propriedade=u'Id', raiz=self.raiz_tag)
        self.IdentificacaoRps = tcIdentificacaoRps(nome=u'IdentificacaoRps', raiz=str_raiz)
        self.DataEmissao = TagData(nome=u'DataEmissao', codigo=u'DataEmissao', raiz=str_raiz)
        self.Status = tsStatusRps(nome=u'Status', codigo=u'Status', raiz=str_raiz)
        self.RpsSubstituido = tcIdentificacaoRps(nome=u'RpsSubstituido', raiz=str_raiz)

    def get_xml(self):
        if not self.IdentificacaoRps.Numero.valor and not self.DataEmissao.valor and \
                self.Status.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.IdentificacaoRps.xml)
        xml.append(self.DataEmissao.xml)
        xml.append(self.Status.xml)
        if self.RpsSubstituido.xml:
            xml.append(self.RpsSubstituido.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.IdentificacaoRps.xml = arquivo
            self.DataEmissao.xml = arquivo
            self.Status.xml = arquivo
            self.RpsSubstituido.xml = arquivo

    xml = property(get_xml, set_xml)

class tcInfDeclaracaoPrestacaoServico(XMLNFSe):
    """
    Representa dados do da declaração do prestador do serviço
    """
    def __init__(self, raiz=''):
        super(tcInfDeclaracaoPrestacaoServico, self).__init__()
        self.nome_tag = 'InfDeclaracaoPrestacaoServico'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'InfDeclaracaoPrestacaoServico',
                              codigo=u'Id', propriedade=u'Id', raiz=self.raiz_tag)
        self.Rps = tcInfRps(raiz=str_raiz)
        self.Competencia = TagData(nome=u'Competencia', codigo=u'Competencia', raiz=str_raiz)
        self.Servico = tcDadosServico(raiz=str_raiz)
        self.Prestador = tcIdentificacaoPrestador(nome=u'Prestador', raiz=str_raiz)
        self.Tomador = tcDadosTomador(raiz=str_raiz)
        self.Intermediario = tcDadosIntermediario(raiz=str_raiz)
        self.ConstrucaoCivil = tcDadosConstrucaoCivil(raiz=str_raiz)
        self.RegimeEspecialTributacao = tsRegimeEspecialTributacao(nome=u'RegimeEspecialTributacao',
                               codigo=u'RegimeEspecialTributacao', raiz=str_raiz,
                                                                   obrigatorio=False)
        self.OptanteSimplesNacional = tsSimNao(nome=u'OptanteSimplesNacional',
                                               codigo=u'OptanteSimplesNacional', raiz=str_raiz)
        self.IncentivoFiscal = tsSimNao(nome=u'IncentivoFiscal', codigo=u'IncentivoFiscal',
                                        raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.Rps.xml)
        xml.append(self.Competencia.xml)
        xml.append(self.Servico.xml)
        xml.append(self.Prestador.xml)
        xml.append(self.Tomador.xml)
        xml.append(self.Intermediario.xml)
        if self.ConstrucaoCivil:
            xml.append(self.ConstrucaoCivil.xml)

        xml.append(self.RegimeEspecialTributacao.xml)
        xml.append(self.OptanteSimplesNacional.xml)
        xml.append(self.IncentivoFiscal.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.Rps.xml = arquivo
            self.Competencia.xml = arquivo
            self.Servico.xml = arquivo
            self.Prestador.xml = arquivo
            self.Tomador.xml = arquivo
            self.Intermediario.xml = arquivo
            self.ConstrucaoCivil.xml = arquivo
            self.RegimeEspecialTributacao.xml = arquivo
            self.OptanteSimplesNacional.xml = arquivo
            self.IncentivoFiscal.xml = arquivo

    xml = property(get_xml, set_xml)

class tcDeclaracaoPrestacaoServico(XMLNFSe):
    """
    Representa a estrutura da declaração da prestação do serviço assinada
    """
    def __init__(self, nome='Rps', raiz=''):
        super(tcDeclaracaoPrestacaoServico, self).__init__()
        self.nome_tag = nome
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.InfDeclaracaoPrestacaoServico = tcInfDeclaracaoPrestacaoServico(raiz=str_raiz)
        self.Signature = Signature()

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.InfDeclaracaoPrestacaoServico.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.InfDeclaracaoPrestacaoServico.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class tcIdentificacaoNfse(XMLNFSe):
    """
    Representa dados que identificam uma Nota Fiscal de Serviços Eletrônica
    """
    def __init__(self, raiz=''):
        super(tcIdentificacaoNfse, self).__init__()
        self.nome_tag = 'IdentificacaoNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Numero = tsNumeroNfse(nome=u'Numero', codigo=u'Numero', raiz=str_raiz)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome=u'InscricaoMunicipal',
                                                       codigo=u'InscricaoMunicipal',
                                                       raiz=str_raiz, obrigatorio=False)
        self.CodigoMunicipio = tsCodigoMunicipioIbge(nome=u'CodigoMunicipio',
                                                     codigo=u'CodigoMunicipio', raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Numero.xml)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(self.CodigoMunicipio.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Numero.xml = arquivo
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo
            self.CodigoMunicipio.xml = arquivo

    xml = property(get_xml, set_xml)

class tcInfNfse(XMLNFSe):
    """
    Representa os dados informativos da Nota Fiscal de Serviços Eletrônica
    """
    def __init__(self, raiz=''):
        super(tcInfNfse, self).__init__()
        self.nome_tag = 'InfNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'NFE', codigo=u'Id', propriedade=u'Id', raiz=str_raiz)
        self.Numero = tsNumeroNfse(nome=u'Numero', codigo=u'Numero', raiz=str_raiz)
        self.CodigoVerificacao = tsCodigoVerificacao(nome=u'CodigoVerificacao',
                                                     codigo=u'CodigoVerificacao', raiz=str_raiz)
        self.DataEmissao = TagDataHora(nome=u'DataEmissao', codigo=u'DataEmissao', raiz=str_raiz)
        self.NfseSubstituida = tsNumeroNfse(nome=u'NfseSubstituida', codigo=u'NfseSubstituida',
                                            raiz=str_raiz, obrigatorio=False)
        self.OutrasInformacoes = tsOutrasInformacoes(nome=u'OutrasInformacoes',
                                                     codigo=u'OutrasInformacoes', raiz=str_raiz,
                                                     obrigatorio=False)
        self.ValoresNfse = tcValoresNfse(codigo=u'ValoresNfse', raiz=str_raiz)
        self.ValorCredito = tsValor(nome=u'ValorCredito', codigo=u'ValorCredito',
                                            raiz=str_raiz, obrigatorio=False)
        self.PrestadorServico = tcDadosPrestador(nome=u'PrestadorServico',
                                                 codigo=u'PrestadorServico', raiz=str_raiz)
        self.OrgaoGerador = tcIdentificacaoOrgaoGerador(codigo=u'OrgaoGerador', raiz=str_raiz)
        self.DeclaracaoPrestacaoServico = tcDeclaracaoPrestacaoServico(
                           nome=u'DeclaracaoPrestacaoServico', codigo=u'DeclaracaoPrestacaoServico',
                           raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.Numero.xml)
        xml.append(self.CodigoVerificacao.xml)
        xml.append(self.DataEmissao.xml)
        xml.append(self.NfseSubstituida.xml)
        xml.append(self.OutrasInformacoes.xml)
        xml.append(self.ValoresNfse.xml)
        xml.append(self.ValorCredito.xml)
        xml.append(self.PrestadorServico.xml)
        xml.append(self.OrgaoGerador.xml)
        xml.append(self.DeclaracaoPrestacaoServico.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.Numero.xml = arquivo
            self.CodigoVerificacao.xml = arquivo
            self.DataEmissao.xml = arquivo
            self.NfseSubstituida.xml = arquivo
            self.OutrasInformacoes.xml = arquivo
            self.ValoresNfse.xml = arquivo
            self.ValorCredito.xml = arquivo
            self.PrestadorServico.xml = arquivo
            self.OrgaoGerador.xml = arquivo
            self.DeclaracaoPrestacaoServico.xml = arquivo

    xml = property(get_xml, set_xml)

class tcNfse(XMLNFSe):
    """
    Representa a estrutura da Nota Fiscal de Serviços Eletrônica assinada
    """
    def __init__(self, raiz=''):
        super(tcNfse, self).__init__()
        self.nome_tag = 'Nfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.versao = TagDecimal(nome=u'Nfse', codigo=u'Nfse', propriedade=u'versao',
                                 raiz=str_raiz, valor=u'1.00', namespace=NAMESPACE_NFSE)
        self.InfNfse = tcInfNfse(codigo=u'InfNfse', raiz=str_raiz)
        self.Signature = Signature()

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s versao="%s">' % (self.nome_tag, self.versao.valor))
        xml.append(self.InfNfse.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.InfNfse.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class tcInfPedidoCancelamento(XMLNFSe):
    """
    Representa a estrutura de dados do pedido de cancelamento enviado pelo prestador ao cancelar
     uma Nota Fiscal de Serviços Eletrônica.
    """
    def __init__(self, raiz=''):
        super(tcInfPedidoCancelamento, self).__init__()
        self.nome_tag = 'InfPedidoCancelamento'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'NFE', codigo=u'Id', propriedade=u'Id', raiz=str_raiz)
        self.IdentificacaoNfse = tcIdentificacaoNfse(codigo=u'IdentificacaoNfse', raiz=str_raiz)
        self.CodigoCancelamento = tsCodigoCancelamentoNfse(nome=u'CodigoCancelamento',
                                                           codigo=u'CodigoCancelamento',
                                                           raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.IdentificacaoNfse.xml)
        xml.append(self.CodigoCancelamento.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.IdentificacaoNfse.xml = arquivo
            self.CodigoCancelamento.xml = arquivo

    xml = property(get_xml, set_xml)

class tcPedidoCancelamento(XMLNFSe):
    """
    Representa a estrutura de Pedido de Cancelamento da Nota Fiscal de Serviços Eletrônica assinada
    """
    def __init__(self, raiz=''):
        super(tcPedidoCancelamento, self).__init__()
        self.nome_tag = 'Pedido'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.InfPedidoCancelamento = tcInfPedidoCancelamento(raiz=str_raiz)
        self.Signature = Signature()

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.InfPedidoCancelamento.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.InfPedidoCancelamento.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class tcConfirmacaoCancelamento(XMLNFSe):
    """
    Representa a estrutura de Confirmação de Cancelamento da Nota Fiscal de Serviços Eletrônica 
    assinada
    """
    def __init__(self, raiz=''):
        super(tcConfirmacaoCancelamento, self).__init__()
        self.nome_tag = 'Confirmacao'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'CONf', codigo=u'Id', propriedade=u'Id', raiz=str_raiz)
        self.Pedido = tcPedidoCancelamento(codigo=u'Pedido', raiz=str_raiz)
        self.DataHora = TagDataHora(nome=u'DataHora', codigo=u'DataHora', raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.Pedido.xml)
        xml.append(self.DataHora.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.Pedido.xml = arquivo
            self.DataHora.xml = arquivo

    xml = property(get_xml, set_xml)

class tcCancelamentoNfse(XMLNFSe):
    """
    Representa a estrutura completa (pedido + confirmação) de cancelamento de NFS-e
    """
    def __init__(self, raiz=''):
        super(tcCancelamentoNfse, self).__init__()
        self.nome_tag = 'NfseCancelamento'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.versao = TagDecimal(nome=u'NfseCancelamento', codigo=u'versao', propriedade=u'versao',
                                 raiz=str_raiz, valor=u'1.00', namespace=NAMESPACE_NFSE)
        self.InfNfse = tcInfNfse(codigo=u'InfNfse', raiz=str_raiz)
        self.Signature = Signature()

    def get_xml(self):
        if not self.InfNfse.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s versao="%s">' % (self.nome_tag, self.versao.valor))
        xml.append(self.InfNfse.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.InfNfse.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class tcRetCancelamento(XMLNFSe):
    """
    Representa a estrutura de Confirmação de Cancelamento da Nota Fiscal de Serviços Eletrônica 
    assinada
    """
    def __init__(self, raiz=''):
        super(tcRetCancelamento, self).__init__()
        self.nome_tag = 'RetCancelamento'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.NfseCancelamento = tcCancelamentoNfse(codigo=u'NfseCancelamento', raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.NfseCancelamento.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NfseCancelamento.xml = arquivo

    xml = property(get_xml, set_xml)

class tcInfSubstituicaoNfse(XMLNFSe):
    """
    Representa os dados de registro de substituição de NFS-e.
    """
    def __init__(self, raiz=''):
        super(tcInfSubstituicaoNfse, self).__init__()
        self.nome_tag = 'SubstituicaoNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'SubsNfse', codigo=u'Id', propriedade=u'Id', raiz=str_raiz)
        self.NfseSubstituidora = tsNumeroNfse(nome=u'NfseSubstituidora',
                                              codigo=u'NfseSubstituidora', raiz=str_raiz)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s Id="%s">' % (self.nome_tag, self.Id.valor))
        xml.append(self.NfseSubstituidora.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.NfseSubstituidora.xml = arquivo

    xml = property(get_xml, set_xml)

class tcSubstituicaoNfse(XMLNFSe):
    """
    Representa a estrutura de substituição de NFS-e.
    """
    def __init__(self, raiz=''):
        super(tcSubstituicaoNfse, self).__init__()
        self.nome_tag = 'NfseSubstituicao'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.versao = TagDecimal(nome=u'NfseCancelamento', codigo=u'versao', propriedade=u'versao',
                                 raiz=str_raiz, valor=u'1.00', namespace=NAMESPACE_NFSE)
        self.SubstituicaoNfse = tcInfSubstituicaoNfse(codigo=u'SubstituicaoNfse', raiz=str_raiz)
        self.Signature = Signature()

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s versao="%s">' % (self.nome_tag, self.versao.valor))
        xml.append(self.SubstituicaoNfse.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.SubstituicaoNfse.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class tcCompNfse(XMLNFSe):
    """
    Representa a estrutura de compartilhamento de dados de uma NFS-e.
    """

    def __init__(self, raiz=''):
        super(tcCompNfse, self).__init__()
        self.nome_tag = 'CompNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Nfse = tcNfse(codigo=u'Nfse', raiz=str_raiz)
        self.NfseCancelamento = tcCancelamentoNfse(codigo=u'NfseCancelamento', raiz=str_raiz,
                                                   obrigatorio=False)
        self.NfseSubstituicao = tcSubstituicaoNfse(codigo=u'NfseSubstituicao', raiz=str_raiz,
                                                   obrigatorio=False)

    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Nfse.xml)
        xml.append(self.NfseCancelamento.xml)
        xml.append(self.NfseSubstituicao.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Nfse.xml = arquivo
            self.NfseCancelamento.xml = arquivo
            self.NfseSubstituicao.xml = arquivo

    xml = property(get_xml, set_xml)

class tcMensagemRetorno(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """

    def __init__(self, raiz=''):
        super(tcMensagemRetorno, self).__init__()
        self.nome_tag = 'MensagemRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Codigo = tsCodigoMensagemAlerta(nome=u'Codigo', codigo=u'Codigo', raiz=str_raiz)
        self.Mensagem = tsDescricaoMensagemAlerta(nome=u'Mensagem', codigo=u'Mensagem',
                                                  raiz=str_raiz)
        self.Correcao = tsDescricaoMensagemAlerta(nome=u'Correcao', codigo=u'Correcao',
                                                  raiz=str_raiz, obrigatorio=False)

    def get_xml(self):
        if not self.Mensagem.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.Codigo.xml)
        xml.append(self.Mensagem.xml)
        xml.append(self.Correcao.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Codigo.xml = arquivo
            self.Mensagem.xml = arquivo
            self.Correcao.xml = arquivo

    xml = property(get_xml, set_xml)

class tcMensagemRetornoLote(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """

    def __init__(self, raiz=''):
        super(tcMensagemRetornoLote, self).__init__()
        self.nome_tag = 'MensagemRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.IdentificacaoRps = tcIdentificacaoRps(nome=u'IdentificacaoRps',
                                                   codigo=u'IdentificacaoRps', raiz=str_raiz)
        self.Codigo = tsCodigoMensagemAlerta(nome=u'Codigo', codigo=u'Codigo', raiz=str_raiz)
        self.Mensagem = tsDescricaoMensagemAlerta(nome=u'Mensagem', codigo=u'Mensagem',
                                                  raiz=str_raiz)

    def get_xml(self):
        if not self.Codigo.valor and not self.Mensagem.valor:
            return u''

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        if self.IdentificacaoRps.xml:
            xml.append(self.IdentificacaoRps.xml)

        xml.append(self.Codigo.xml)
        xml.append(self.Mensagem.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.IdentificacaoRps.xml = arquivo
            self.Codigo.xml = arquivo
            self.Mensagem.xml = arquivo

    xml = property(get_xml, set_xml)

class tcLoteRps(XMLNFSe):
    """
    Representa a estrutura do lote de RPS para fila de processamento
    """
    def __init__(self, raiz=''):
        super(tcLoteRps, self).__init__()
        self.nome_tag = 'LoteRps'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.Id = TagCaracter(nome=u'LoteRps', codigo=u'Id', propriedade=u'Id', raiz=self.raiz_tag)
        self.versao = TagDecimal(nome=u'LoteRps', codigo=u'versao', propriedade=u'versao',
                                 raiz=self.raiz_tag, valor=u'1.00', namespace=NAMESPACE_NFSE)
        self.NumeroLote = tsNumeroLote(nome=u'NumeroLote', codigo=u'NumeroLote', raiz=str_raiz)
        self.CpfCnpj = tcCpfCnpj(raiz=str_raiz)
        self.InscricaoMunicipal = tsInscricaoMunicipal(nome=u'InscricaoMunicipal',
                                                       codigo=u'InscricaoMunicipal',
                                                       raiz=self.raiz_tag,
                                                       obrigatorio=False)
        self.QuantidadeRps = tsQuantidadeRps(nome=u'QuantidadeRps', codigo=u'QuantidadeRps',
                                             raiz=str_raiz)
        self.Rps = [] # tcInfDeclaracaoPrestacaoServico

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s Id="%s" versao="%s">' % (self.nome_tag, self.Id.valor, self.versao.valor))
        xml.append(self.NumeroLote.xml)
        xml.append(self.CpfCnpj.xml)
        xml.append(self.InscricaoMunicipal.xml)
        xml.append(self.QuantidadeRps.xml)
        xml.append(u'<ListaRps>')
        for i in self.Rps:
            xml.append(i.xml)

        xml.append(u'</ListaRps>')
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml = arquivo
            self.versao.xml = arquivo
            self.NumeroLote.xml = arquivo
            self.CpfCnpj.xml = arquivo
            self.InscricaoMunicipal.xml = arquivo
            self.QuantidadeRps.xml = arquivo
            self.Rps = self.le_grupo('%s/%s/ListaRps' % (self.raiz_tag, self.nome_tag),
                                     tcDeclaracaoPrestacaoServico, arquivo)

    xml = property(get_xml, set_xml)

class ListaMensagemRetornoLote(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """
    def __init__(self, raiz=''):
        super(ListaMensagemRetornoLote, self).__init__()
        self.nome_tag = 'ListaMensagemRetornoLote'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.MensagemRetorno = []

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        for i in self.MensagemRetorno:
            xml.append(i.xml)

        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.MensagemRetorno = self.le_grupo('//%s/%s/MensagemRetorno' % (self.raiz_tag,
                                                                              self.nome_tag),
                                                 tcMensagemRetornoLote)

    xml = property(get_xml, set_xml)

class ListaMensagemRetorno(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """
    def __init__(self, raiz=''):
        super(ListaMensagemRetorno, self).__init__()
        self.nome_tag = 'ListaMensagemRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.MensagemRetorno = tcMensagemRetorno(raiz=str_raiz)

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.MensagemRetorno.xml)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.MensagemRetorno.xml = arquivo

    xml = property(get_xml, set_xml)

class ListaMensagemAlertaRetorno(XMLNFSe):
    """
    Representa a estrutura de mensagem de retorno de serviço.
    """
    def __init__(self, raiz=''):
        super(ListaMensagemAlertaRetorno, self).__init__()
        self.nome_tag = 'ListaMensagemAlertaRetorno'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.MensagemRetorno = tcMensagemRetorno(raiz=str_raiz)

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.MensagemRetorno.xml)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.MensagemRetorno.xml = arquivo

    xml = property(get_xml, set_xml)

class CompNfse(XMLNFSe):
    """
    Representa a estrutura da NFS-e.
    """
    def __init__(self, raiz=''):
        super(CompNfse, self).__init__()
        self.nome_tag = 'CompNfse'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.CompNfse = tcCompNfse(codigo=u'CompNfse', raiz=str_raiz)

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]

        xml.append(u'<%s>' % self.nome_tag)
        xml.append(self.CompNfse.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CompNfse.xml   = arquivo

    xml = property(get_xml, set_xml)



# ----------------------------Servicos---------------------------------


class EnviarLoteRpsEnvio(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsEnvio, self).__init__()
        self.nome_tag = 'EnviarLoteRpsEnvio'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.LoteRps = tcLoteRps(raiz=str_raiz)
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/')
        self.arquivo_esquema = u'nfse_v202.xsd'

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<?xml version="1.0"?>')
        xml.append(u'<%s %s>' % (self.nome_tag, u'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.LoteRps.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.LoteRps.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class EnviarLoteRpsResposta(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsResposta, self).__init__()
        self.nome_tag = 'EnviarLoteRpsResposta'
        self.raiz_tag = raiz
        str_raiz = '%s/%s' % (raiz, self.nome_tag)
        self.NumeroLote = tsNumeroLote(nome=u'NumeroLote', codigo=u'NumeroLote', raiz=str_raiz)
        self.DataRecebimento = TagDataHora(nome=u'DataRecebimento', codigo=u'DataRecebimento',
                                           raiz=str_raiz)
        self.Protocolo = tsNumeroProtocolo(nome=u'Protocolo', codigo=u'Protocolo', raiz=str_raiz)
        self.ListaMensagemRetorno = ListaMensagemRetorno(raiz=str_raiz)
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/')
        self.arquivo_esquema = u'nfse_v202.xsd'

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<?xml version="1.0"?>')
        xml.append(u'<%s %s>' % (self.nome_tag,  u'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.NumeroLote.xml)
        xml.append(self.DataRecebimento.xml)
        xml.append(self.Protocolo.xml)
        xml.append(self.ListaMensagemRetorno.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NumeroLote.xml = arquivo
            self.DataRecebimento.xml = arquivo
            self.Protocolo.xml = arquivo
            self.ListaMensagemRetorno.xml = arquivo

    xml = property(get_xml, set_xml)

class EnviarLoteRpsSincronoEnvio(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsSincronoEnvio, self).__init__()
        self.nome_tag = 'EnviarLoteRpsSincronoEnvio'
        self.raiz_tag = raiz
        str_raiz = '//%s%s' % (raiz, self.nome_tag)
        self.LoteRps = tcLoteRps(raiz=str_raiz)
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/')
        self.arquivo_esquema = u'nfse_v202.xsd'


    def get_xml(self):
        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<?xml version="1.0"?>')
        xml.append(u'<%s %s>' % (self.nome_tag, u'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.LoteRps.xml)
        if len(self.Signature.URI) and (self.Signature.URI.strip() != u'#'):
            xml.append(self.Signature.xml)

        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.LoteRps.xml = arquivo
            self.Signature.xml = self._le_noh('//%s/sig:Signature' % self.nome_tag)

    xml = property(get_xml, set_xml)

class EnviarLoteRpsSincronoResposta(XMLNFSe):
    def __init__(self, raiz=''):
        super(EnviarLoteRpsSincronoResposta, self).__init__()
        self.nome_tag = 'EnviarLoteRpsSincronoResposta'
        self.raiz_tag = raiz
        str_raiz = '//%s%s' % (raiz, self.nome_tag)
        self.NumeroLote = tsNumeroLote(nome=u'NumeroLote', codigo=u'NumeroLote', raiz=str_raiz)
        self.DataRecebimento = TagDataHora(nome=u'DataRecebimento', codigo=u'DataRecebimento',
                                           raiz=str_raiz)
        self.Protocolo = tsNumeroProtocolo(nome=u'Protocolo', codigo=u'Protocolo', raiz=str_raiz)
        self.CompNfse = []
        self.ListaMensagemAlertaRetorno = ListaMensagemAlertaRetorno(raiz=str_raiz)
        self.ListaMensagemRetorno = ListaMensagemRetorno(raiz=str_raiz)
        self.ListaMensagemRetornoLote = ListaMensagemRetornoLote(raiz=str_raiz)
        self.caminho_esquema = os.path.join(DIRNAME, u'schema/')
        self.arquivo_esquema = u'nfse_v202.xsd'

    def get_xml(self):

        xml = [XMLNFSe.get_xml(self)]
        xml.append(u'<?xml version="1.0"?>')
        xml.append(u'<%s %s>' % (self.nome_tag, u'xmlns="%s"' % NAMESPACE_NFSE))
        xml.append(self.NumeroLote.xml)
        xml.append(self.DataRecebimento.xml)
        xml.append(self.Protocolo.xml)
        xml.append(u'<ListaNfse>')
        for i in self.CompNfse:
            xml.append(i.xml)

        xml.append(self.ListaMensagemAlertaRetorno.xml)
        xml.append(u'</ListaNfse>')
        xml.append(self.ListaMensagemRetorno.xml)
        xml.append(self.ListaMensagemRetornoLote.xml)
        xml.append(u'</%s>' % self.nome_tag)
        return u''.join(xml)

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NumeroLote.xml = arquivo
            self.DataRecebimento.xml = arquivo
            self.Protocolo.xml = arquivo
            self.ListaNfse = self.le_grupo('%s/%s/ListaNfse' % (self.raiz_tag, self.nome_tag),
                                           tcCompNfse, arquivo)
            self.ListaMensagemRetorno.xml = arquivo
            self.ListaMensagemRetornoLote.xml = arquivo

    xml = property(get_xml, set_xml)